/*****************************************************************************************/
/*BEGINPROLOGUE************************************************************************
 * @copyright(disclaimer)                                                             *
 * Licensed Materials - Property of IBM                                               *
 * IBM Enterprise Information Portal for Multiplatforms V8.1 (program number 5724-B43)*
 * (c ) Copyright IBM Corp. 1998,  2002. All Rights Reserved.                         *
 *                                                                                    *
 * US Government Users Restricted Rights                                              *
 * Use, duplication or disclosure restricted by GSA ADP Schedule                      *
 * Contract with IBM Corp.                                                            *
 *                                                                                    *
 * DISCLAIMER OF WARRANTIES :                                                         *
 *                                                                                    *
 * Permission is granted to copy and modify this Sample code, and to                  *
 * distribute modified versions provided that both the copyright                      *
 * notice,- and this permission notice and warranty disclaimer appear in all copies   * 
 * and modified versions.                                                             *
 *                                                                                    *
 * THIS SAMPLE CODE IS LICENSED TO YOU AS-IS.                                         *
 * IBM AND ITS SUPPLIERS AND LICENSORS DISCLAIM ALL WARRANTIES, EITHER EXPRESS OR     *
 * IMPLIED, IN SUCH SAMPLE CODE, INCLUDING THE WARRANTY OF NON-INFRINGEMENT AND THE   *
 * IMPLIED WARRANTIES OF MERCHANTABILITY OR FITNESS FOR A PARTICULAR PURPOSE. IN NO   *
 * EVENT WILL IBM OR ITS LICENSORS OR SUPPLIERS BE LIABLE FOR ANY DAMAGES ARISING OUT *
 * OF THE USE OF OR INABILITY TO USE THE SAMPLE CODE, DISTRIBUTION OF THE SAMPLE CODE,*
 * OR COMBINATION OF THE SAMPLE CODE WITH ANY OTHER CODE. IN NO EVENT SHALL IBM OR ITS*
 * LICENSORS AND SUPPLIERS BE LIABLE FOR ANY LOST REVENUE, LOST PROFITS OR DATA, OR   *
 * FOR DIRECT, INDIRECT, SPECIAL, CONSEQUENTIAL,INCIDENTAL OR PUNITIVE DAMAGES,       *
 * HOWEVER CAUSED AND REGARDLESS OF THE THEORY OF LIABILITY,-, EVEN IF IBM OR ITS     *
 * LICENSORS OR SUPPLIERS HAVE BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGES.       *
 *                                                                                    *
 * @endCopyright                                                                      *
 **************************************************************************************/
/*****************************************************************************************/

// sample to change a user password on an OD server
/*
 * Filename       : TChangePasswordOD.cpp
 * Description    : test DKDatastoreOD for change password function
 * Change History :
 *
 * Date        Name  Description
 * ----------  -----  ---------------------------
 * 2001/08/10  nnc   Creation of test case
 *
 */

#include <iostream.h>
#include <stdio.h>
#include <DKConstant.h>
#include <DKDatastoreOD.hpp>
#include <DKException.hpp>
#include <DKString.hpp>
#include <DKEnvironment.hpp>

void main(int argc, char *argv[])
{
  int num_loops = 1;

  cout << "begin..." << endl;
  // Indicate this is a Console Application on Windows NT/95 for error
  // message code page conversion to OEM code page. If this was a Windows
  // GUI (Graphic User Interface) Application this would not be necessary
  // because DK_SS_WINDOWS is the default.
  DKEnvironment::setSubSystem(DK_SS_CONSOLE);

  //--------------------------------------------------------------
  // Checking for input parameters
  //--------------------------------------------------------------
  if (argc < 4)
  {
    cout<<"Usage: "<<endl;
    cout<<"  TChangePasswordOD <libsrv> <userid> <old pw> <new pw> <connect_string>" << endl;
    return;
  }
  // ### get input parameters
  DKString libsrv = DKString(argv[1]);
  DKString userid = DKString(argv[2]);
  DKString pwd    = DKString(argv[3]);

  DKString new_pwd        = "";
  DKString connect_string = "";
  if (argc > 4)
    new_pwd = argv[4];   // new pwd
  if (argc > 5)
    connect_string = argv[5];

  // must be outside try/catch block
  DKDatastoreOD *dsOD = NULL;

  try
  {
    dsOD = new DKDatastoreOD();
    DKString userName = dsOD->userName();
    DKString dsName = dsOD->datastoreName();
    cout << "before connecting to datastore" << endl;
    cout << "  parameters returned from datastore - user: " <<
                       userName << "; dsName: " << dsName << endl;
    cout << "connecting to datastore ..." << endl;
    dsOD->connect(libsrv,userid,pwd,connect_string);
    cout << "  connection ok" << endl;
    cout << "  verifying parameters ..." << endl;
    cout << "    arguments received from command    - user: " <<
                       userid << "; library server: " << libsrv << endl;
    userName = dsOD->userName();
    dsName = dsOD->datastoreName();
    cout << "    parameters returned from datastore - user: " <<
                       userName << "; dsName:         " << dsName << endl;
    //-----------------------------------------------------------------------------
    //    Change password
    //-----------------------------------------------------------------------------
    dsOD->changePassword(userid, pwd, new_pwd);
    cout << "password has been changed to new password: " << new_pwd << endl;

    dkDatastoreDef*   dsDef = dsOD->datastoreDef();
    dkDatastoreAdmin* dsAdmin = dsDef->datastoreAdmin();
    dkUserManagement* userMgt = (dkUserManagement*)dsAdmin->userManagement();
    if (userMgt != NULL)
    {
      userMgt->changePassword(userid,new_pwd,pwd);
      cout << "password has been changed back using the dkUserManagement object."<< endl;
    }
    else
    {
      cout << "dkUserManagement is NULL!" << endl;
    }
    //dsOD->changePassword(userid, new_pwd, pwd);
    //cout << "password has been restored to old password: " << pwd << endl;
    cout << "  -password change has executed successfully!" << endl;

    dsOD->disconnect();
    cout << "disconnecting from the datastore ..." << endl;
    userName = dsOD->userName();
    dsName = dsOD->datastoreName();
    cout << "  disconnection ok" << endl;
    cout << "  parameters returned from datastore - user: " <<
                       userName << "; dsName: " << dsName << endl;
    delete dsOD;
  }
  catch (DKException &exc)
  {
    cout << "Error id " << exc.errorId() << endl;
    for (unsigned long i = 0; i< exc.textCount();i++)
    {
      cout << "Error text: " << exc.text(i) << endl;
    }
    for (unsigned long g=0; g< exc.locationCount();g++)
    {
      const DKExceptionLocation* p = exc.locationAtIndex(g);
      cout << "Filename: " << p->fileName() << endl;
      cout << "Function: " << p->functionName() << endl;
      cout << "LineNumber: " << p->lineNumber() << endl;
    }

    cout << "Exception Class Name: " << exc.name() << endl;
  }

  cout << "done ..." << endl;
}

