/*****************************************************************************************/
/*BEGINPROLOGUE************************************************************************
 * @copyright(disclaimer)                                                             *
 * Licensed Materials - Property of IBM                                               *
 * IBM Enterprise Information Portal for Multiplatforms V8.1 (program number 5724-B43)*
 * (c ) Copyright IBM Corp. 1998,  2002. All Rights Reserved.                         *
 *                                                                                    *
 * US Government Users Restricted Rights                                              *
 * Use, duplication or disclosure restricted by GSA ADP Schedule                      *
 * Contract with IBM Corp.                                                            *
 *                                                                                    *
 * DISCLAIMER OF WARRANTIES :                                                         *
 *                                                                                    *
 * Permission is granted to copy and modify this Sample code, and to                  *
 * distribute modified versions provided that both the copyright                      *
 * notice,- and this permission notice and warranty disclaimer appear in all copies   * 
 * and modified versions.                                                             *
 *                                                                                    *
 * THIS SAMPLE CODE IS LICENSED TO YOU AS-IS.                                         *
 * IBM AND ITS SUPPLIERS AND LICENSORS DISCLAIM ALL WARRANTIES, EITHER EXPRESS OR     *
 * IMPLIED, IN SUCH SAMPLE CODE, INCLUDING THE WARRANTY OF NON-INFRINGEMENT AND THE   *
 * IMPLIED WARRANTIES OF MERCHANTABILITY OR FITNESS FOR A PARTICULAR PURPOSE. IN NO   *
 * EVENT WILL IBM OR ITS LICENSORS OR SUPPLIERS BE LIABLE FOR ANY DAMAGES ARISING OUT *
 * OF THE USE OF OR INABILITY TO USE THE SAMPLE CODE, DISTRIBUTION OF THE SAMPLE CODE,*
 * OR COMBINATION OF THE SAMPLE CODE WITH ANY OTHER CODE. IN NO EVENT SHALL IBM OR ITS*
 * LICENSORS AND SUPPLIERS BE LIABLE FOR ANY LOST REVENUE, LOST PROFITS OR DATA, OR   *
 * FOR DIRECT, INDIRECT, SPECIAL, CONSEQUENTIAL,INCIDENTAL OR PUNITIVE DAMAGES,       *
 * HOWEVER CAUSED AND REGARDLESS OF THE THEORY OF LIABILITY,-, EVEN IF IBM OR ITS     *
 * LICENSORS OR SUPPLIERS HAVE BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGES.       *
 *                                                                                    *
 * @endCopyright                                                                      *
 **************************************************************************************/
/*****************************************************************************************/

// test list servers/ list schema / list schema attributes from OD

/*
 * Filename       : TListFolderCriteriaOD.java
 * Description    : test DKDatastoreOD for list functions
 * Change History :
 *
 * Date        Name  Description
 * -------    -----  ---------------------------
 * 2001/01/02  bkh   Creation
 * 2001/06/21  nnc   Ported to C++
 */

#include <iostream.h>
#include <stdio.h>
#include <DKConstant.h>
#include <DKDatastoreOD.hpp>
#include <DKServerDefOD.hpp>
#include <DKDatastoreDefOD.hpp>
#include <DKException.hpp>
#include <DKString.hpp>
#include <DKEnvironment.hpp>

static DKString Op2Str(short Op);

void main(int argc, char *argv[])
{
  cout << "begin..." << endl;
  // Indicate this is a Console Application on Windows NT/95 for error
  // message code page conversion to OEM code page. If this was a Windows
  // GUI (Graphic User Interface) Application this would not be necessary
  // because DK_CM_SS_WINDOWS is the default.
  DKEnvironment::setSubSystem(DK_CM_SS_CONSOLE);

  //--------------------------------------------------------------
  // Checking for input parameters
  //--------------------------------------------------------------
  if (argc < 4)
  {
    cout << "Usage: " << endl;
    cout << "  TListFolderCriteriaODOD <libsrv> <userid> <pw> <connect_string> <configuration_string>" << endl;
    return;
  }
  // ### get input parameters
  DKString libsrv = DKString(argv[1]);
  DKString userid = DKString(argv[2]);
  DKString pw = DKString(argv[3]);
  DKString connect_string = "";
  DKString configuration_string = "";
  if (argc > 4 )
    connect_string = argv[4];
  if (argc > 5 )
    configuration_string = argv[5];

  // must be outside try/catch block
  DKDatastoreOD* dsOD;

  if (configuration_string != "")
  {
    dsOD  = new DKDatastoreOD(configuration_string);
  }
  else
  {
    dsOD  = new DKDatastoreOD();
  }
  cout << "Datastore OD created" << endl;
  cout << "connect_string=>" << connect_string << "<=" << endl;
  cout << "configuration string=>" << configuration_string << "<=" << endl;

  DKSequentialCollection * pCol = NULL;
  dkIterator * pIter = NULL;
  DKSequentialCollection * pCol2 = NULL;
  DKServerDefOD * pSV = NULL;
  DKDatastoreDefOD * dsDef = NULL;

  try
  {
    DKString strServerName;
    DKString strServerType;
    DKString folderName;
    int i = 0;
    int j = 0;

    cout << "list servers" << endl;
    pCol = (DKSequentialCollection*)dsOD->listDataSources();
    if (pCol)
    {
      pIter = pCol->createIterator();
      if (pIter)
      {
        while (pIter->more() == TRUE)
        {
          i++;
          pSV = (DKServerDefOD*)((void*)(*pIter->next()));
          strServerName = pSV->getName();
          strServerType = pSV->getServerType();
          cout << "Server Name [" << i << "] - " << strServerName << " Server Type - " << strServerType << endl;
          delete pSV;
        }
        cout << i << " servers listed" << endl;
        delete pIter;
        pIter = NULL;
      }
      delete pCol;
      pCol = NULL;
    }

    cout << "connecting to datastore" << endl;
    dsOD->connect(libsrv,userid,pw,connect_string);
    cout << "datastore connected libsrv=" << libsrv << ", userid=" << userid << endl;
    cout << "connect_string=" << connect_string << endl;
    cout << endl;

    cout << "list folders" << endl;
    dsDef = (DKDatastoreDefOD*)dsOD->datastoreDef();
    pCol = (DKSequentialCollection*)dsDef->listSearchTemplates();
    pIter = pCol->createIterator();
    i = 0;

    while (pIter->more() == TRUE)
    {
      i++;
      DKFolderOD* flddef = (DKFolderOD*)(void*)(*pIter->next());
      folderName = flddef->getName();
      cout << "\nfolder name [" << i << "] - " << folderName << endl;
      pCol2 = (DKSequentialCollection*)flddef->listCriteria();
      dkIterator* pIter2 = pCol2->createIterator();
      j = 0;
      while (pIter2->more() == TRUE)
      {
        j++;
        DKSearchCriteria* critdef = (DKSearchCriteria*)(void*)(*pIter2->next());
        DKString critName = critdef->getName();
        cout << "\n  Criteria #" << j << ": name=>" << critName << "<=" << endl;

        // valid operators
        long arraySize = 0;
        short* validOperators = critdef->getValidOperators(arraySize);
        if (validOperators != NULL)
        {
          for (int k = 0; k < arraySize; k++)
          {
            cout <<"   Valid Operator[" << (k+1) << "]=" << "'" << Op2Str(validOperators[k]) << "'" << endl;
          }
          delete [] validOperators;
        }
        else
        {
          cout << "   No valid operators." << endl;
        }
        //default operator
        cout <<"   Default operator='" << Op2Str(critdef->getDefaultOperator()) << "'" << endl;

        // default values
        DKString* defaultValues = critdef->getDefaultValues(arraySize);
        if (defaultValues != NULL)
        {
          for (int k = 0;  k < arraySize; k++)
          {
            cout << "   Default Value[" << (k+1) << "]=>" << defaultValues[k] << "<=" << endl;
          }
          delete [] defaultValues;
        }
        else
        {
          cout <<"   **No Default Values**" << endl;
        }

        // predefined values
        dkCollection* predefinedValues = critdef->getPredefinedValues();
        dkIterator* pIter3 = predefinedValues->createIterator();
        int k = 0;
        int l = 0;
        while (pIter3->more() == TRUE)
        {
          k++;
          //cout << "   Predefined Values set #" << k << ":" << endl;
          DKPredefinedValues* predefValues = (DKPredefinedValues*)(void*)(*pIter3->next());
          dkCollection* predefValPairs = predefValues->getValuePairs();
          dkIterator* pIter4 = predefValPairs->createIterator();
          while (pIter4->more() == TRUE)
          {
            l++;
            DKNVPair* vp = (DKNVPair*)(void*)(*pIter4->next());
            DKString name = vp->getName();
            cout << "   Predefined Value[" << l << "]: =>" << name << "<=" << endl;
          }
          if (predefValPairs)
            delete predefValPairs;
          if (pIter4)
            delete pIter4;
        }
        if (predefinedValues)
          delete predefinedValues;
        if (pIter3)
          delete pIter3;
        if (l==0)
        {
          cout << "   **No Predefined Values**" << endl;
        }
      }
      if ( pCol2 )
        delete pCol2;
      if (pIter2)
        delete pIter2;
    }
    cout << i << " folders listed" << endl;

    dsOD->disconnect();
    cout << "datastore disconnected" << endl;
  }
  catch (DKException &exc)
  {
    cout << "Error id " << exc.errorId() << endl;
    for (unsigned long i = 0; i< exc.textCount();i++)
    {
      cout << "Error text: " << exc.text(i) << endl;
    }
    for (unsigned long g=0; g< exc.locationCount();g++)
    {
      const DKExceptionLocation* p = exc.locationAtIndex(g);
      cout << "Filename: " << p->fileName() << endl;
      cout << "Function: " << p->functionName() << endl;
      cout << "LineNumber: " << p->lineNumber() << endl;
    }

    cout << "Exception Class Name: " << exc.name() << endl;
  }

  // cleanup
  if ( pCol )
    delete pCol;
  if ( pIter )
    delete pIter;
  if ( dsOD )
    delete dsOD;

  cout << "done ..." << endl;
}

//***********************************************************************
//   Op2Str
//***********************************************************************
static DKString Op2Str(short Op)
{
  DKString OpStr = "";
  switch (Op)
  {
    case DK_CM_OPCODE_EQ         : OpStr = "EQ"         ; break;
    case DK_CM_OPCODE_NEQ        : OpStr = "NEQ"        ; break;
    case DK_CM_OPCODE_GT         : OpStr = "GT"         ; break;
    case DK_CM_OPCODE_LT         : OpStr = "LT"         ; break;
    case DK_CM_OPCODE_LIKE       : OpStr = "LIKE"       ; break;
    case DK_CM_OPCODE_NOTLIKE    : OpStr = "NOTLIKE"    ; break;
    case DK_CM_OPCODE_BETWEEN    : OpStr = "BETWEEN"    ; break;
    case DK_CM_OPCODE_IN         : OpStr = "IN"         ; break;
    case DK_CM_OPCODE_NOTIN      : OpStr = "NOTIN"      ; break;
    case DK_CM_OPCODE_GEQ        : OpStr = "GEQ"        ; break;
    case DK_CM_OPCODE_LEQ        : OpStr = "LEQ"        ; break;
    case DK_CM_OPCODE_NOTBETWEEN : OpStr = "NOTBETWEEN" ; break;
    case DK_CM_OPCODE_AND        : OpStr = "AND"        ; break;
    case DK_CM_OPCODE_OR         : OpStr = "OR"         ; break;
    case DK_CM_OPCODE_NOT        : OpStr = "NOT"        ; break;
    case DK_CM_OPCODE_UNDEFINED  : OpStr = "UNDEFINED"  ; break;
    default                      : OpStr = "Unknown!!!" ; break;
  }
  return OpStr;
}

