/*****************************************************************************************/
/*BEGINPROLOGUE************************************************************************
 * @copyright(disclaimer)                                                             *
 * Licensed Materials - Property of IBM                                               *
 * IBM Enterprise Information Portal for Multiplatforms V8.1 (program number 5724-B43)*
 * (c ) Copyright IBM Corp. 1998,  2002. All Rights Reserved.                         *
 *                                                                                    *
 * US Government Users Restricted Rights                                              *
 * Use, duplication or disclosure restricted by GSA ADP Schedule                      *
 * Contract with IBM Corp.                                                            *
 *                                                                                    *
 * DISCLAIMER OF WARRANTIES :                                                         *
 *                                                                                    *
 * Permission is granted to copy and modify this Sample code, and to                  *
 * distribute modified versions provided that both the copyright                      *
 * notice,- and this permission notice and warranty disclaimer appear in all copies   * 
 * and modified versions.                                                             *
 *                                                                                    *
 * THIS SAMPLE CODE IS LICENSED TO YOU AS-IS.                                         *
 * IBM AND ITS SUPPLIERS AND LICENSORS DISCLAIM ALL WARRANTIES, EITHER EXPRESS OR     *
 * IMPLIED, IN SUCH SAMPLE CODE, INCLUDING THE WARRANTY OF NON-INFRINGEMENT AND THE   *
 * IMPLIED WARRANTIES OF MERCHANTABILITY OR FITNESS FOR A PARTICULAR PURPOSE. IN NO   *
 * EVENT WILL IBM OR ITS LICENSORS OR SUPPLIERS BE LIABLE FOR ANY DAMAGES ARISING OUT *
 * OF THE USE OF OR INABILITY TO USE THE SAMPLE CODE, DISTRIBUTION OF THE SAMPLE CODE,*
 * OR COMBINATION OF THE SAMPLE CODE WITH ANY OTHER CODE. IN NO EVENT SHALL IBM OR ITS*
 * LICENSORS AND SUPPLIERS BE LIABLE FOR ANY LOST REVENUE, LOST PROFITS OR DATA, OR   *
 * FOR DIRECT, INDIRECT, SPECIAL, CONSEQUENTIAL,INCIDENTAL OR PUNITIVE DAMAGES,       *
 * HOWEVER CAUSED AND REGARDLESS OF THE THEORY OF LIABILITY,-, EVEN IF IBM OR ITS     *
 * LICENSORS OR SUPPLIERS HAVE BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGES.       *
 *                                                                                    *
 * @endCopyright                                                                      *
 **************************************************************************************/
/*****************************************************************************************/

// sample connect/execute/retrieve/disconnect to OD
/*
 * Filename       : TRetrieveFolderWithCallbackOD.cpp
 * Description    : Connect to OD Datastore.
 *                  Create query string.
 *                  Specify to retrieve documents.
 *                  Specify to retrieve a max result of 5 items.
 *                  Get the result in a resultset cursor.
 *                  Loop thru results.
 *                  Display documents found and their parts and attributes,
 *                   if folders found display them and their documents/
 *                   folders.
 *                  Disconnect from OD Datastore.
 *
 *                  configuration string can be
 *                      CC2MIMEURL=(File:///c:\cmbroot\cmbcc2mime.ini);
 *                    or
 *                      CC2MIMEURL=(HTTP://www.my.com/cmbcc2mime.ini);
 *                    or
 *                      CC2MIMEFILE=(c:\cmbroot\cmbcc2mime.ini);
 */

#ifdef WIN32
#include <io.h>
#include <conio.h>
#endif
#include <iostream.h>
#include <stdio.h>
#include <DKConstant.h>
#include <DKDatastoreOD.hpp>
#include <DKException.hpp>
#include <DKString.hpp>
#include <dkResultSetCursor.hpp>
#include <DKDDO.hpp>
#include <DKAny.hpp>
#include <DKPid.hpp>
#include <dkCollection.hpp>
#include <DKBlobOD.hpp>
#include <DKPidXDOOD.hpp>
#include <DKEnvironment.hpp>
#include "TCallbackOD.hpp"

static DKBoolean CompareIgnoreCase(DKString str1, DKString str2);

static DKString Op2Str(short Op);

static void setFolderQueryCriteria(int testcase, DKString libsrv);

static DKString folderName;

static long      NumCriteria = 0;
static DKString* CriteriaNames = 0;
static DKString* CriteriaValues = 0;
static short*    CriteriaOperators = 0;

void main(int argc, char *argv[])
{
  cout << "begin..." << endl;

  // Indicate this is a Console Application on Windows NT/95 for error
  // message code page conversion to OEM code page. If this was a Windows
  // GUI (Graphic User Interface) Application this would not be necessary
  // because DK_CM_SS_WINDOWS is the default.
  DKEnvironment::setSubSystem(DK_CM_SS_CONSOLE);

  long threshold    = 10;
  long iterations   = 2;
  BOOL retrieveHits = TRUE;

  //--------------------------------------------------------------
  // Checking for input parameters
  //--------------------------------------------------------------
  if (argc < 4)
  {
    cout<<"Usage: "<<endl;
    cout<<"  TRetrieveFolderWithCallbackOD <libsrv> <userid> <pw> <connect_string> <configuration string> threshold iterations" << endl;
    return;
  }
  // ### get input parameters
  DKString libsrv = DKString(argv[1]);
  DKString userid = DKString(argv[2]);
  DKString pw = DKString(argv[3]);
  DKString connect_string = "";
  DKString configuration_string = "";
  DKString threshold_string, iterations_string;
  if (argc > 4 )
    connect_string = argv[4];
  if (argc > 5 )
    configuration_string = argv[5];
  if (argc > 6 )
  {
    threshold_string = argv[6];
    threshold  = threshold_string.asInt();
  }
  if (argc > 7 )
  {
    iterations_string  = argv[7];
    iterations = iterations_string.asInt();
  }

  DKDatastoreOD* dsOD; // must be outside try/catch block

  if (configuration_string != "")
  {
    configuration_string += ";ENTITY_TYPE=TEMPLATES";
    dsOD = new DKDatastoreOD(configuration_string);
  }
  else
  {
    dsOD = new DKDatastoreOD("ENTITY_TYPE=TEMPLATES");
  }
  cout << "Datastores dsOD created" << endl;
  cout << "connect_string=>" << connect_string << "<=" << endl;
  cout << "configuration string=>" << configuration_string << "<=" << endl;
  cout << "threshold=" << threshold << endl;
  cout << "iterations=" << iterations << endl;

  DKFolderOD* folder = NULL;
  DKSearchCriteria** criteria = NULL;

  try
  {
    dkResultSetCursor* pCur = 0;

    //@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
    // Select a testcase by changing this number!!!
    int testcase = 2;
    //@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
    setFolderQueryCriteria(testcase, libsrv);

    cout << "\nUsing folder =>" << folderName << "<=" << endl;

    cout << "connecting to datastore" << endl;
    dsOD->connect(libsrv,userid,pw,connect_string);
    cout << "datastore connected " << libsrv << " userid - " << userid << endl;

    DKNVPair parms[3];
    parms[0] = DKNVPair("MAX_RESULTS", DKString(threshold*iterations));
    parms[1] = DKNVPair("CONTENT", DKString("ATTRONLY"));
    parms[2] = DKNVPair( DK_CM_PARM_END, DKAny((long)0) );

    DKTimestamp timeBefore;
    cout << "Before query, time stamp is " << timeBefore.asString(NULL) << endl;

    // List folder names
    DKDatastoreDefOD* dsDef = (DKDatastoreDefOD*)dsOD->datastoreDef();
    dkCollection* folders = dsDef->listSearchTemplates();
    dkIterator* pIter = folders->createIterator();
    DKFolderOD* fldOD;
    int i = 0;
    DKBoolean foundIt = FALSE;
    while (pIter->more() == TRUE)
    {
      i++;
      fldOD = (DKFolderOD*)(void*)(*pIter->next());
      DKString thisFolderName = fldOD->getName();
      cout << "folder[" << i << "]: " << thisFolderName << endl;
      if (thisFolderName == folderName)
      {
        folder = fldOD;
        foundIt = TRUE;
        break;
      }
    }
    cout << "\nUsing folder =>" << folderName << "<=" << endl;

    if (foundIt)
    {
      // Set the search criteria
      criteria = folder->getCritInfo();
      int ii = 0;
      int numQueryCrit = folder->getNumQueryCrit();
      for (int k = 0; k < numQueryCrit; k++)
      {
        DKString critName = criteria[k]->getName();
        if (critName == CriteriaNames[ii])
        {
          criteria[k]->setValue(CriteriaValues[ii]);
          criteria[k]->setOperator(CriteriaOperators[ii]);
          cout << "  Name: " << critName << endl;
          cout << "  =>value: " << CriteriaValues[ii] << endl;
          cout << "  =>operator: " << Op2Str(CriteriaOperators[ii]) << endl;
          ii++;
        }
        if (ii >= NumCriteria)
        {
          break;
        }
      }

      TCallbackOD callback(dsOD,
                           threshold,
                           iterations,
                           retrieveHits); // whether to save hits for retrieval
      cout << "***Threshold is set to " << threshold << ", iterations is set to " << iterations << endl;
      cout << "executing query with callback using datastore" << endl;
      dsOD->executeWithCallback(folder,parms,&callback);
      cout << "query executed using datastore" << endl;
      DKTimestamp timeAfter;
      cout << "After query, time stamp is " << timeAfter.asString(NULL) << endl;
      while (!callback.isDone())
      {
  #ifdef WIN32
        Sleep(1000);    // 1000ms
  #else
        usleep(10000);  // 10000 microseconds
  #endif
      }

      // check for exception
      DKException* excObj = callback.getException();
      if (excObj != 0)
      {
         cout << "Error found " << excObj->text(0) << endl;
         delete excObj;
         callback.resetException();
      }

      DKTimestamp timeAfter2;
      cout << "\nAfter results processed " << timeAfter2.asString(NULL) << endl;
      cout << "\nquery results processed" << endl;
      if (pCur != 0)
        delete pCur;

      // retrieve and display saved hits (DDOs)
      if (retrieveHits)
      {
        cout << "" << endl;
        cout << "================================" << endl;
        cout << "==the retrieval process begins==" << endl;
        cout << "================================" << endl;
        callback.retrieveDDO();
      }
    }

    dsOD->disconnect();
    cout << "\ndatastore disconnected" << endl;
    delete dsOD;
    delete folders;
    delete pIter;

    if (CriteriaNames != 0)
      delete [] CriteriaNames;
    if (CriteriaValues != 0)
      delete [] CriteriaValues;
    if (CriteriaOperators != 0)
      delete [] CriteriaOperators;
  }

  catch (DKException &exc)
  {

    cout << "Error id " << exc.errorId() << endl;
    cout << "Exception id " << exc.exceptionId() << endl;
    for (unsigned long i = 0; i< exc.textCount();i++)
    {
      cout << "Error text: " << exc.text(i) << endl;
    }
    for (unsigned long g=0; g< exc.locationCount();g++)
    {
      const DKExceptionLocation* p = exc.locationAtIndex(g);
      cout << "Filename: " << p->fileName() << endl;
      cout << "Function: " << p->functionName() << endl;
      cout << "LineNumber: " << p->lineNumber() << endl;
    }
    cout << "Exception Class Name: " << exc.name() << endl;
  }
  cout << "TRetrieveFolderWithCallbackOD done ..." << endl;
}

//***********************************************************************
//   Op2Str
//***********************************************************************
static DKString Op2Str(short Op)
{
  DKString OpStr = "";
  switch (Op)
  {
    case DK_CM_OPCODE_EQ         : OpStr = "EQ"         ; break;
    case DK_CM_OPCODE_NEQ        : OpStr = "NEQ"        ; break;
    case DK_CM_OPCODE_GT         : OpStr = "GT"         ; break;
    case DK_CM_OPCODE_LT         : OpStr = "LT"         ; break;
    case DK_CM_OPCODE_LIKE       : OpStr = "LIKE"       ; break;
    case DK_CM_OPCODE_NOTLIKE    : OpStr = "NOTLIKE"    ; break;
    case DK_CM_OPCODE_BETWEEN    : OpStr = "BETWEEN"    ; break;
    case DK_CM_OPCODE_IN         : OpStr = "IN"         ; break;
    case DK_CM_OPCODE_NOTIN      : OpStr = "NOTIN"      ; break;
    case DK_CM_OPCODE_GEQ        : OpStr = "GEQ"        ; break;
    case DK_CM_OPCODE_LEQ        : OpStr = "LEQ"        ; break;
    case DK_CM_OPCODE_NOTBETWEEN : OpStr = "NOTBETWEEN" ; break;
    case DK_CM_OPCODE_AND        : OpStr = "AND"        ; break;
    case DK_CM_OPCODE_OR         : OpStr = "OR"         ; break;
    case DK_CM_OPCODE_NOT        : OpStr = "NOT"        ; break;
    case DK_CM_OPCODE_UNDEFINED  : OpStr = "UNDEFINED"  ; break;
    default                      : OpStr = "Unknown!!!" ; break;
  }
  return OpStr;
}

// --------------------------------------------------------------------
// Compare two strings case insensitively
// --------------------------------------------------------------------
static DKBoolean CompareIgnoreCase(DKString str1, DKString str2)
{
#ifdef WIN32
  return (str1.CompareNoCase(str2) == 0);
#else
  return (IString::lowerCase(str1) == IString::lowerCase(str2));
#endif
}

// --------------------------------------------------------------------
// Set the folder query criteria
// --------------------------------------------------------------------
static void setFolderQueryCriteria(int testcase, DKString libsrv)
{
  // Folder and search criteria
  // OD/390 v2.1
  // Folder and search criteria
  if (CompareIgnoreCase(libsrv, "9.91.121.31"))
  {
    NumCriteria = 3;
    CriteriaNames     = new DKString[NumCriteria];
    CriteriaValues    = new DKString[NumCriteria];
    CriteriaOperators = new short[NumCriteria];

    switch (testcase)
    {
      default:
      case 1:  // AFP
        folderName = "CHKSTMN1";
        CriteriaNames[0] = "ACCOUNT NUMBER";
        break;
      case 2:
        folderName = "COMPAT2";
        CriteriaNames[0] = "ACCOUNT NUMBER";
        break;
      case 3:
        folderName = "COMPATD";
        CriteriaNames[0] = "ACCOUNT NUMBER";
        break;
      case 4:
        folderName = "DELINQCY";
        CriteriaNames[0] = "LOAN NUMBER";
        break;
      case 5:
        folderName = "FSINV";
        CriteriaNames[0] = "INVOICE#";
        break;
      case 6:
        folderName = "INSURE";
        CriteriaNames[0] = "POLICY NUMBER";
        break;
      case 7:
        folderName = "INSURTST";
        CriteriaNames[0] = "POLICY NUMBER";
        break;
      case 8:
        folderName = "K0560";
        CriteriaNames[0] = "FIL#";
        break;
      case 9:
        folderName = "KEYBANK";
        CriteriaNames[0] = "ACCOUNT NUMBER";
        break;
    }

    CriteriaValues   [0] = "%";
    CriteriaOperators[0] = DK_CM_OPCODE_EQ;
    CriteriaNames    [1] = "FROM (YYYY-MM-DD)";
    CriteriaValues   [1] = "1991-05-26";
    CriteriaOperators[1] = DK_CM_OPCODE_GEQ;
    CriteriaNames    [2] = "TO   (YYYY-MM-DD)";
    CriteriaValues   [2] = "2001-05-23";
    CriteriaOperators[2] = DK_CM_OPCODE_LEQ;
  }
  // OD/390 v7.1
  else if (CompareIgnoreCase(libsrv, "9.91.121.34"))
  {
    switch (testcase)
    {
      default:
      case 1: // AFP data
        folderName = "Credit Card Statements";

        NumCriteria = 1;
        CriteriaNames     = new DKString[NumCriteria];
        CriteriaValues    = new DKString[NumCriteria];
        CriteriaOperators = new short[NumCriteria];

        CriteriaNames    [0] = "Account";
        CriteriaValues   [0] = "%"; //"000-000-010";
        CriteriaOperators[0] = DK_CM_OPCODE_LIKE;
        break;
      case 2:
        folderName = "CHKSTMNT";

        NumCriteria = 1;
        CriteriaNames     = new DKString[NumCriteria];
        CriteriaValues    = new DKString[NumCriteria];
        CriteriaOperators = new short[NumCriteria];

        CriteriaNames    [0] = "ACCOUNT NUM";
        CriteriaValues   [0] = "%";
        CriteriaOperators[0] = DK_CM_OPCODE_LIKE;
        break;
      case 3:  // line data
        folderName = "Delinquency";

        NumCriteria = 1;
        CriteriaNames     = new DKString[NumCriteria];
        CriteriaValues    = new DKString[NumCriteria];
        CriteriaOperators = new short[NumCriteria];

        CriteriaNames    [0] = "Page Number";
        CriteriaValues   [0] = "0";
        CriteriaOperators[0] = DK_CM_OPCODE_NEQ;
        break;
      case 4:  // line data large object
        folderName = "LOAN LO";

        NumCriteria = 1;
        CriteriaNames     = new DKString[NumCriteria];
        CriteriaValues    = new DKString[NumCriteria];
        CriteriaOperators = new short[NumCriteria];

        CriteriaNames    [0] = "Date";
        CriteriaValues   [0] = "10/01/94";
        CriteriaOperators[0] = DK_CM_OPCODE_EQ;
        break;
      case 5:  // pdf
        folderName = "PDF Landscape";

        NumCriteria = 1;
        CriteriaNames     = new DKString[NumCriteria];
        CriteriaValues    = new DKString[NumCriteria];
        CriteriaOperators = new short[NumCriteria];

        CriteriaNames    [0] = "Region";
        CriteriaValues   [0] = "'%'";
        CriteriaOperators[0] = DK_CM_OPCODE_LIKE;
      case 6:  // AFP
        folderName = "afpdata";

        NumCriteria = 1;
        CriteriaNames     = new DKString[NumCriteria];
        CriteriaValues    = new DKString[NumCriteria];
        CriteriaOperators = new short[NumCriteria];

        CriteriaNames    [0] = "Tax Year";
        CriteriaValues   [0] = "1995";
        CriteriaOperators[0] = DK_CM_OPCODE_EQ;
        break;
    }
  }
  // OD/400 v4.5
  else if (libsrv.length() >= 7 &&
           CompareIgnoreCase(libsrv.subString(1,7),"rdr400g"))
  {
    switch (testcase)
    {
      default:
      case 1:  // AFP
        folderName = "CheckStatements";

        NumCriteria = 1;
        CriteriaNames     = new DKString[NumCriteria];
        CriteriaValues    = new DKString[NumCriteria];
        CriteriaOperators = new short[NumCriteria];

        CriteriaNames    [0] = "Customer Name";
        CriteriaValues   [0] = "%";
        CriteriaOperators[0] = DK_CM_OPCODE_LIKE;
        break;
    }
  }
  else if (libsrv.length() >= 8 &&
           CompareIgnoreCase(libsrv.subString(1,8),"stealth5"))
  {
    folderName = "Credit Card Statements";

    NumCriteria = 1;
    CriteriaNames     = new DKString[NumCriteria];
    CriteriaValues    = new DKString[NumCriteria];
    CriteriaOperators = new short[NumCriteria];

    CriteriaNames    [0] = "Name";
    CriteriaValues   [0] = "%";
    CriteriaOperators[0] = DK_CM_OPCODE_LIKE;
  }
  else if (libsrv.length() >= 6 &&
           CompareIgnoreCase(libsrv.subString(1,6),"reagan"))
  {
    folderName = "load-db2-CRD";

    NumCriteria = 1;
    CriteriaNames     = new DKString[NumCriteria];
    CriteriaValues    = new DKString[NumCriteria];
    CriteriaOperators = new short[NumCriteria];

    CriteriaNames    [0] = "Account Name";
    CriteriaValues   [0] = "%";
    CriteriaOperators[0] = DK_CM_OPCODE_LIKE;
  }
  else if (libsrv.length() >= 6 &&
           CompareIgnoreCase(libsrv.subString(1,6),"reagan"))
  {
    switch (testcase)
    {
      default:
      case 1:  // AFP
        folderName = "load-db2-CRD";

        NumCriteria = 1;
        CriteriaNames     = new DKString[NumCriteria];
        CriteriaValues    = new DKString[NumCriteria];
        CriteriaOperators = new short[NumCriteria];

        CriteriaNames    [0] = "Account Name";
        CriteriaValues   [0] = "%";
        CriteriaOperators[0] = DK_CM_OPCODE_LIKE;
        break;
    }
  }
  else if (libsrv.length() >= 7 &&
           CompareIgnoreCase(libsrv.subString(1,8),"everest"))
  {
    switch (testcase)
    {
      default:
      case 1:  // PDF
        folderName = "43MBPDF";

        NumCriteria = 1;
        CriteriaNames     = new DKString[NumCriteria];
        CriteriaValues    = new DKString[NumCriteria];
        CriteriaOperators = new short[NumCriteria];

        CriteriaNames    [0] = "Report Title";
        CriteriaValues   [0] = "%";
        CriteriaOperators[0] = DK_CM_OPCODE_LIKE;
        break;
    }
  }
  else if (libsrv.length() >= 5 &&
           CompareIgnoreCase(libsrv.subString(1,5),"akira"))
  {
    switch (testcase)
    {
      default:
      case 1:  // AFP
        folderName = "rl-DFAC";

        NumCriteria = 1;
        CriteriaNames     = new DKString[NumCriteria];
        CriteriaValues    = new DKString[NumCriteria];
        CriteriaOperators = new short[NumCriteria];

        CriteriaNames    [0] = "nomcli"; //"exploit";
        CriteriaValues   [0] = "MR PEREZ ERIC"; //"%";
        CriteriaOperators[0] = DK_CM_OPCODE_LIKE;
        break;
      case 2:
        folderName = "LARpmr94348";

        NumCriteria = 1;
        CriteriaNames     = new DKString[NumCriteria];
        CriteriaValues    = new DKString[NumCriteria];
        CriteriaOperators = new short[NumCriteria];

        CriteriaNames    [0] = "Customer Id"; ;
        CriteriaValues   [0] = "%";
        CriteriaOperators[0] = DK_CM_OPCODE_LIKE;
        break;
    }
  }
  else //if (CompareIgnoreCase(libsrv.subString(1,6),"gunnar"))
  {
    // Single object AFP
    switch (testcase)
    {
      default:
      case 1:  // AFP
        folderName = "Credit Card Statements";

        NumCriteria = 1;
        CriteriaNames     = new DKString[NumCriteria];
        CriteriaValues    = new DKString[NumCriteria];
        CriteriaOperators = new short[NumCriteria];

        CriteriaNames    [0] = "Account";
        CriteriaValues   [0] = "%"; //"000-000-010";
        CriteriaOperators[0] = DK_CM_OPCODE_LIKE;
        break;
    }
  }
}

