/*****************************************************************************************/
/*BEGINPROLOGUE************************************************************************
 * @copyright(disclaimer)                                                             *
 * Licensed Materials - Property of IBM                                               *
 * IBM Enterprise Information Portal for Multiplatforms V8.1 (program number 5724-B43)*
 * (c ) Copyright IBM Corp. 1998,  2002. All Rights Reserved.                         *
 *                                                                                    *
 * US Government Users Restricted Rights                                              *
 * Use, duplication or disclosure restricted by GSA ADP Schedule                      *
 * Contract with IBM Corp.                                                            *
 *                                                                                    *
 * DISCLAIMER OF WARRANTIES :                                                         *
 *                                                                                    *
 * Permission is granted to copy and modify this Sample code, and to                  *
 * distribute modified versions provided that both the copyright                      *
 * notice,- and this permission notice and warranty disclaimer appear in all copies   * 
 * and modified versions.                                                             *
 *                                                                                    *
 * THIS SAMPLE CODE IS LICENSED TO YOU AS-IS.                                         *
 * IBM AND ITS SUPPLIERS AND LICENSORS DISCLAIM ALL WARRANTIES, EITHER EXPRESS OR     *
 * IMPLIED, IN SUCH SAMPLE CODE, INCLUDING THE WARRANTY OF NON-INFRINGEMENT AND THE   *
 * IMPLIED WARRANTIES OF MERCHANTABILITY OR FITNESS FOR A PARTICULAR PURPOSE. IN NO   *
 * EVENT WILL IBM OR ITS LICENSORS OR SUPPLIERS BE LIABLE FOR ANY DAMAGES ARISING OUT *
 * OF THE USE OF OR INABILITY TO USE THE SAMPLE CODE, DISTRIBUTION OF THE SAMPLE CODE,*
 * OR COMBINATION OF THE SAMPLE CODE WITH ANY OTHER CODE. IN NO EVENT SHALL IBM OR ITS*
 * LICENSORS AND SUPPLIERS BE LIABLE FOR ANY LOST REVENUE, LOST PROFITS OR DATA, OR   *
 * FOR DIRECT, INDIRECT, SPECIAL, CONSEQUENTIAL,INCIDENTAL OR PUNITIVE DAMAGES,       *
 * HOWEVER CAUSED AND REGARDLESS OF THE THEORY OF LIABILITY,-, EVEN IF IBM OR ITS     *
 * LICENSORS OR SUPPLIERS HAVE BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGES.       *
 *                                                                                    *
 * @endCopyright                                                                      *
 **************************************************************************************/
/*****************************************************************************************/

// sample connect/execute/retrieve/disconnect to OD
/*
 * Filename       : TRetrieveWithCallbackOD.cpp
 * Description    : Connect to OD Datastore.
 *                  Create query string.
 *                  Specify to retrieve folder and documents.
 *                  Specify to retrieve a max result of 5 items.
 *                  Get the result in a resultset cursor.
 *                  Loop thru results.
 *                  Display documents found and their parts and attributes,
 *                   if folders found display them and their documents/
 *                   folders.
 *                  Disconnect from OD Datastore.
 *
 *                  configuration string can be
 *                      CC2MIMEURL=(File:///c:\cmbroot\cmbcc2mime.ini);
 *                    or
 *                      CC2MIMEURL=(HTTP://www.my.com/cmbcc2mime.ini);
 *                    or
 *                      CC2MIMEFILE=(c:\cmbroot\cmbcc2mime.ini);
 */

#ifdef WIN32
#include <io.h>
#include <conio.h>
#endif
#include <iostream.h>
#include <stdio.h>
#include <DKConstant.h>
#include <DKDatastoreOD.hpp>
#include <DKException.hpp>
#include <DKString.hpp>
#include <dkResultSetCursor.hpp>
#include <DKDDO.hpp>
#include <DKAny.hpp>
#include <DKPid.hpp>
#include <dkCollection.hpp>
#include <DKBlobOD.hpp>
#include <DKPidXDOOD.hpp>
#include <DKEnvironment.hpp>
#include "TCallbackOD.hpp"

static DKBoolean CompareIgnoreCase(DKString str1, DKString str2);
static DKString ODdate( int month, int day, int year );
static void setAGQueryCriteria(int testcase, DKString libSrv);

static DKString appgrp = "";
static DKString cmd    = "";

void main(int argc, char *argv[])
{
  cout << "begin..." << endl;

  // Indicate this is a Console Application on Windows NT/95 for error
  // message code page conversion to OEM code page. If this was a Windows
  // GUI (Graphic User Interface) Application this would not be necessary
  // because DK_CM_SS_WINDOWS is the default.
  DKEnvironment::setSubSystem(DK_CM_SS_CONSOLE);

  long MaxCount     = -1;
  long threshold    = 10;
  long iterations   = 2;
  BOOL retrieveHits = TRUE;

  //--------------------------------------------------------------
  // Checking for input parameters
  //--------------------------------------------------------------
  if (argc < 4)
  {
    cout<<"Usage: "<<endl;
    cout<<"  TRetrieveWithCallbackOD <libsrv> <userid> <pw> <connect_string> <configuration string> threshold iterations" << endl;
    return;
  }
  // ### get input parameters
  DKString libsrv = DKString(argv[1]);
  DKString userid = DKString(argv[2]);
  DKString pw = DKString(argv[3]);
  DKString connect_string = "";
  DKString configuration_string = "";
  DKString threshold_string, iterations_string;
  if (argc > 4 )
    connect_string = argv[4];
  if (argc > 5 )
    configuration_string = argv[5];
  if (argc > 6 )
  {
    threshold_string = argv[6];
    threshold  = threshold_string.asInt();
  }
  if (argc > 7 )
  {
    iterations_string  = argv[7];
    iterations = iterations_string.asInt();
  }

  DKDatastoreOD* dsOD; // must be outside try/catch block

  if (configuration_string != "")
  {
    dsOD = new DKDatastoreOD(configuration_string);
  }
  else
  {
    dsOD = new DKDatastoreOD();
  }
  cout << "Datastores dsOD created" << endl;
  cout << "connect_string=>" << connect_string << "<=" << endl;
  cout << "configuration string=>" << configuration_string << "<=" << endl;
  cout << "threshold=" << threshold << endl;
  cout << "iterations=" << iterations << endl;

  try
  {
    dkResultSetCursor* pCur = 0;

    //@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
    // Select a testcase by changing this number!!!
    int testcase = 10;
    //@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
    setAGQueryCriteria(testcase, libsrv);

    DKNVPair parms[4];
    parms[0] = DKNVPair("APPL_GROUP", appgrp);
    parms[1] = DKNVPair("MAX_RESULTS", DKString(MaxCount));
    parms[2] = DKNVPair("CONTENT", DKString("ATTRONLY"));
    parms[3] = DKNVPair( DK_CM_PARM_END, DKAny((long)0) );

    cout << "\nApplication group =>" << appgrp << "<=" << endl;
    cout << "Query string =>" << cmd << "<=\n" << endl;

    cout << "connecting to datastore" << endl;
    dsOD->connect(libsrv,userid,pw,connect_string);
    cout << "datastore connected " << libsrv << " userid - " << userid << endl;

    DKTimestamp timeBefore;
    cout << "Before query, time stamp is " << timeBefore.asString(NULL) << endl;

    // Execute with callback!!!
    TCallbackOD callback(dsOD,
                         threshold,
                         iterations,
                         retrieveHits); // whether to save hits for retrieval
    cout << "***Threshold is set to " << threshold << ", iterations is set to " << iterations << endl;
    cout << "executing query with callback using datastore" << endl;
    dsOD->executeWithCallback(cmd,DK_CM_SQL_QL_TYPE,parms,&callback);
    cout << "query executed using datastore" << endl;
    DKTimestamp timeAfter;
    cout << "After query, time stamp is " << timeAfter.asString(NULL) << endl;
    while (!callback.isDone())
    {
#ifdef WIN32
      Sleep(1000);    // 1000ms
#else
      usleep(10000);  // 10000 microseconds
#endif
    }

    // check for exception
    DKException* excObj = callback.getException();
    if (excObj != 0)
    {
       cout << "Error found " << excObj->text(0) << endl;
       delete excObj;
       callback.resetException();
    }

    DKTimestamp timeAfter2;
    cout << "\nAfter results processed " << timeAfter2.asString(NULL) << endl;
    cout << "\nquery results processed" << endl;
    if (pCur != 0)
      delete pCur;

    // retrieve and display saved hits (DDOs)
    if (retrieveHits)
    {
      cout << "" << endl;
      cout << "================================" << endl;
      cout << "==the retrieval process begins==" << endl;
      cout << "================================" << endl;
      callback.retrieveDDO();
    }

    dsOD->disconnect();
    cout << "\ndatastore disconnected" << endl;
    delete dsOD;
  }

  catch (DKException &exc)
  {

    cout << "Error id " << exc.errorId() << endl;
    cout << "Exception id " << exc.exceptionId() << endl;
    for (unsigned long i = 0; i< exc.textCount();i++)
    {
      cout << "Error text: " << exc.text(i) << endl;
    }
    for (unsigned long g=0; g< exc.locationCount();g++)
    {
      const DKExceptionLocation* p = exc.locationAtIndex(g);
      cout << "Filename: " << p->fileName() << endl;
      cout << "Function: " << p->functionName() << endl;
      cout << "LineNumber: " << p->lineNumber() << endl;
    }
    cout << "Exception Class Name: " << exc.name() << endl;
  }
  cout << "TRetrieveWithCallbackOD done ..." << endl;
}

// --------------------------------------------------------------------
// Convert regular month, day, year values to an OnDemand integer date
// Asumming (and hopefully) the year is in the yyyy format
// --------------------------------------------------------------------
static DKString ODdate( int month, int day, int year )
{
  int days[] = {31,28,31,30,31,30,31,31,30,31,30,31};
  int OD_days = 0;

  // Just in case someone entered a 2 digit year
  if ( year < 100 )
  {
     // 21st century
     if (year < 70)
        year += 2000;
     else
        year += 1900;
  }

  // Get the numbers of days since 1/1/1970 to 1/1/yyyy
  OD_days = ( year - 1970 ) * 365;

  // Add days from leap years
  for ( int yr = 1970; yr < year; yr++ )
  {
    if ((((yr) % 4) == 0) && ((((yr) % 100) != 0) || (((yr) % 400) == 0)))
    {
      OD_days++;
    }
  }

  // Add days sicne 1/1 of this year
  for ( int mon = 0; mon < month-1; mon++ )
  {
    OD_days += days[mon];
  }

  // Add the current day
  OD_days += day;

  // Additional Leap year adjustment
  if ((((year) % 4) == 0) && ((((year) % 100) != 0) || (((year) % 400) == 0)) && month > 2)
    OD_days++;

  DKString od_days_str( OD_days );

  return od_days_str;
}

// --------------------------------------------------------------------
// Compare two strings case insensitively
// --------------------------------------------------------------------
static DKBoolean CompareIgnoreCase(DKString str1, DKString str2)
{
#ifdef WIN32
  return (str1.CompareNoCase(str2) == 0);
#else
  return (IString::lowerCase(str1) == IString::lowerCase(str2));
#endif
}

// --------------------------------------------------------------------
// set application group and query SQL
// --------------------------------------------------------------------
static void setAGQueryCriteria(int testcase, DKString libsrv)
{
  int month = 6;
  int day   = 2;
  int year  = 1997;

  DKString  mon_str( month );
  DKString  day_str( day );
  DKString  yr_str( year );
  DKString  date_str;
  date_str = mon_str + "/" + day_str + "/" + yr_str;
  DKString od_date = ODdate( month, day, year );

  // Application Group and SQL string
  // OD/390 v2.1
  if (CompareIgnoreCase(libsrv, "9.91.121.31"))
  {
    switch (testcase)
    {
      default:
      case 1:  // AFP
        appgrp = "INSURE  01 ";
        cmd = "WHERE (POLICY_NUMBER LIKE '324-1443255%')";  //'324-1443255-11' ";
        break;
      case 2:
        appgrp = "CHKSTMN301 ";
        cmd = "WHERE ACCOUNT_NUMBER LIKE '%' ";
        break;
      case 3:  // line/large object
        appgrp = "DELINQCY02 ";
        cmd = "WHERE ( LOAN_NUMBER LIKE '%' )";
        break;
      case 4:  // pdf
        appgrp = "KEYBANK 01 ";
        cmd = "WHERE ( ACCOUNT_NUMBER LIKE '%' )";
    }
  }
  // OD/390 v7.1
  else if (CompareIgnoreCase(libsrv, "9.91.121.34"))
  {
    switch (testcase)
    {
      default:
      case 1: // AFP data
        appgrp = "Credit Card Statements";
        cmd = "WHERE ( name LIKE '%' )";
        break;
      case 2:
        appgrp = "CHKSTMNT";
        cmd = "WHERE ACCOUNT_NUMBER LIKE '%' ";
        break;
      case 3:  // line data
        appgrp = "Delinqcy-AG";
        cmd = "WHERE ( pagenum != 0 )";
        break;
      case 4:  // line data large object
        appgrp = "LOAN LO";
        od_date = ODdate( 10, 1, 1994 );
        cmd = "WHERE ( load_date = " + od_date + " )";
        //od_date = ODdate( 2, 2, 1970 );
        //cmd  = "WHERE (( load_date > " + od_date + ") AND ";
        //cmd += "(bgn_loan_num <= 10000 AND end_loan_num >= 10000) )";
        break;
      case 5:  // pdf
        appgrp = "PDF landscape";
        cmd = "WHERE ( region LIKE '%' )";
      case 6:  // AFP
        appgrp = "afpdata";
        cmd = "WHERE (TaxYear = 1995)";
        break;
    }
  }
  else if (CompareIgnoreCase(libsrv, "gunnar"))
  {
    switch (testcase)
    {
      default:
      case 1: // Large object AFP - gunnar
        appgrp = "OnDemand Publications";
        cmd = "where bookname LIKE '%'";
        break;
    }
  }
  else if (CompareIgnoreCase(libsrv.subString(1, 7), "everest"))
  {
    switch (testcase)
    {
      default:
      case 1:
        // AFP data
        appgrp = "CRD";
        cmd = "WHERE ( name LIKE '%' )";
        break;
      case 2:
        // Large Object line data
        appgrp = "sp-ebcdic-lo";
        //od_date = ODdate( 1, 1, 1990 );
        //od_date2 = ODdate( 12, 31, 2001 );
        //cmd = "WHERE ( rpt_date BETWEEN " + od_date + " AND " + od_date2 + " )";
        od_date = ODdate( 12, 15, 2000 );
        cmd = "WHERE ( rpt_date > 0 )"; //= " + od_date + " )";
        break;
      case 3: // Loan Delinquency Reports - Large Objects
        appgrp = "LOAN LO";
        od_date = ODdate( 2, 2, 1970 );   // 2/2/1970
        cmd  = "WHERE (( load_date > " + od_date + ") AND ";
        cmd += "(bgn_loan_num <= 10000 AND end_loan_num >= 10000) )";
        break;
      case 4:
        // Large Object line data
        appgrp = "eliz-largeline";
        od_date = ODdate( 3, 19, 1999 );
        cmd = "WHERE ( rptdate = " + od_date + " )";
        break;
      case 5:
        // PDF
        appgrp = "43MBPDF";
        cmd = "WHERE ( rtitle LIKE '%' )";
        break;
    }
  }
  else if (CompareIgnoreCase(libsrv.subString(1, 5), "akira"))
  {
    switch (testcase)
    {
      default:
      case 1: // AFP
        appgrp = "Few Credit Card Statements";
        cmd = "WHERE ( name LIKE '%' )";
        break;
      case 2: // Line - Loan Delinquency Reports
        appgrp = "Loan Delinquency Reports";
        od_date = ODdate( 2, 2, 1970 );   // 2/2/1970
        cmd  = "WHERE (( load_date > " + od_date + ") AND ";
        cmd += "(bgn_loan_num <= 10000 AND end_loan_num >= 10000) )";
        break;
      case 3: // User defined types
        appgrp = "User Defined Data Types";
        cmd = "WHERE ( Docname LIKE '%' )";
        break;
      case 4:
        appgrp = "LARpmr94348z";
        cmd = "WHERE ( cuid LIKE '%' )";
        break;
    }
  }
  // OD/400 v4.5
  else if (CompareIgnoreCase(libsrv.subString(1, 7), "rdr400c"))
  {
    switch (testcase)
    {
      default:
      case 1:
        appgrp = "CHECKSTMTS ";
        cmd = "WHERE ( KY1VAL LIKE '%' )";
        break;
    }
  }
  // OD/400 v5.1
  else if (CompareIgnoreCase(libsrv.subString(1, 7), "rdr400g"))
  {
    switch (testcase)
    {
      default:
      case 1:
        appgrp = "CheckStatements";
        cmd = "WHERE ( name LIKE '%' )";
        break;
    }
  }
  else // can be any server!
  {
    switch (testcase)
    {
      default:
      case 1: // AFP
        // native OD integer date
        od_date = ODdate( month, day, year );
        cout << "The date '" << date_str <<"' equals to the OnDemand integer date '" << od_date<<"'" << endl;

        appgrp = "CRD";
        //cmd = "WHERE ( crd_date = " + od_date + " )";
        cmd = "WHERE ( name LIKE '%')";
        break;
      case 2:  // line
        appgrp = "Balance";
        cmd = "WHERE ( hiercode LIKE '..21...002')"; //'%.....006')";
        break;
    }
  }
}
