/*BEGINPROLOGUE************************************************************************
 * @copyright(disclaimer)                                                             *
 * Licensed Materials - Property of IBM                                               *
 * IBM Information Integrator for Content V8.2  (program number 5724-B43)             *
 * (c ) Copyright IBM Corp. 1998, 2002, 2003. All Rights Reserved.                    *
 *                                                                                    *
 * US Government Users Restricted Rights                                              *
 * Use, duplication or disclosure restricted by GSA ADP Schedule                      *
 * Contract with IBM Corp.                                                            *
 *                                                                                    *
 * DISCLAIMER OF WARRANTIES :                                                         *
 *                                                                                    *
 * Permission is granted to copy and modify this Sample code, and to                  *
 * distribute modified versions provided that both the copyright                      *
 * notice,- and this permission notice and warranty disclaimer appear in all copies   * 
 * and modified versions.                                                             *
 *                                                                                    *
 * THIS SAMPLE CODE IS LICENSED TO YOU AS-IS.                                         *
 * IBM AND ITS SUPPLIERS AND LICENSORS DISCLAIM ALL WARRANTIES, EITHER EXPRESS OR     *
 * IMPLIED, IN SUCH SAMPLE CODE, INCLUDING THE WARRANTY OF NON-INFRINGEMENT AND THE   *
 * IMPLIED WARRANTIES OF MERCHANTABILITY OR FITNESS FOR A PARTICULAR PURPOSE. IN NO   *
 * EVENT WILL IBM OR ITS LICENSORS OR SUPPLIERS BE LIABLE FOR ANY DAMAGES ARISING OUT *
 * OF THE USE OF OR INABILITY TO USE THE SAMPLE CODE, DISTRIBUTION OF THE SAMPLE CODE,*
 * OR COMBINATION OF THE SAMPLE CODE WITH ANY OTHER CODE. IN NO EVENT SHALL IBM OR ITS*
 * LICENSORS AND SUPPLIERS BE LIABLE FOR ANY LOST REVENUE, LOST PROFITS OR DATA, OR   *
 * FOR DIRECT, INDIRECT, SPECIAL, CONSEQUENTIAL,INCIDENTAL OR PUNITIVE DAMAGES,       *
 * HOWEVER CAUSED AND REGARDLESS OF THE THEORY OF LIABILITY,-, EVEN IF IBM OR ITS     *
 * LICENSORS OR SUPPLIERS HAVE BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGES.       *
 *                                                                                    *
 * @endCopyright                                                                      *
 **************************************************************************************

Get Work from a Work List
    A Work List may be used to obtain Work Packages for one or more Work Nodes.  This 
    example demonstrates how to list Work Package information using a Work List.

********************************************************************/

// Includes / Imports
#include <iostream.h>
#include <DKConstant.h>
#include <DKDatastoreICM.hpp>
#include <DKDocRoutingServiceICM.hpp>
#include "SConnectDisconnectICM.hpp"
#include "SDocRoutingListingICM.hpp"

/************************************************************************************************
 *          FILENAME: SDocRoutingListingICM.cpp
 *                    ---------------------------------------------------------------------------
 *       DESCRIPTION: List all Routing Process in System, List Work Nodes, List Work Lists,
 *                    Count number of Packages in a Work List, Obtain all Work Packages in
 *                    a Work List, Retrieve & List WOrk package Details, Print Work Package
 *                    Information.
 *                    ---------------------------------------------------------------------------
 *     DEMONSTRATION: List Routing Processes
 *                    List Work Nodes
 *                    List Work Lists
 *                    Count the number of Packages currently in a Work List
 *                    Obtain all Work Packages in a Work List
 *                    Retrieve & List Work Package Details
 *                    Printing Work Package Information
 *                    ---------------------------------------------------------------------------
 * COMMANDLINE USAGE: SDocRoutingListingICM <database> <userName> <password>
 *                    ---------------------------------------------------------------------------
 *      PREREQUISITS: The Data Model must be defined.  If needed please run the following Samples  
 *                        - SAttributeDefinitionCreationICM
 *                        - SAttributeGroupDefCreationICM
 *                        - SReferenceAttrDefCreationICM
 *                        - SItemTypeCreationICM
 *                    ---------------------------------------------------------------------------
 *    FILES REQUIRED: SConnectDisconnectICM.hpp/cpp
 *                    SDocRoutingDefinitionListingICM_launch.cpp
 *                    SDocRoutingDefinitionListingICM.hpp
 ************************************************************************************************/

    //-------------------------------------------------------------
    // Main
    //-------------------------------------------------------------
    /**
     * Run the Sample.
     * @param argc   Argument Count / Number of Arguments Specified
     * @param argv[] String Array containing arguments.  Optional arguments are <databse> <userName> <password>.
     * @return  Returns SConnectDisconnectICM_RC_SAMPLE_PASSED (0) if no sample program completed, SConnectDisconnectICM_RC_SAMPLE_FAILED (-1) if any errors / exceptions were caught.
     **/
    int SDocRoutingListingICM::main(int argc, char *argv[]){

        // Defaults for connecting to the database.
        char * database = SConnectDisconnectICM_DEFAULT_DATABASE;
        char * userName = SConnectDisconnectICM_DEFAULT_USERNAME;
        char * password = SConnectDisconnectICM_DEFAULT_PASSWORD;

        //------------------------------------------------------------
        // Checking for input parameters
        //--------------------------------------------------------------
        if (argc < 4) { // if not all 3 arguments were specified, use defaults and report correct usage.
            cout << "Usage: "                                                    << endl;
            cout << "  SDocRoutingListingICM <database> <userName> <password>"   << endl;
            cout << "  *** Some parameters not specified, using defaults..."     << endl;
            cout << endl;
        } else {  // otherwise enough parameters were specified, use the first 3 for the 3 parameters.
            if (argc > 1) database = argv[1];
            if (argc > 2) userName = argv[2];
            if (argc > 3) password = argv[3];
        }//end else
        char * ver = SConnectDisconnectICM_EIP_VERSION;

        cout << "==========================================="     << endl;
        cout << "IBM Information Integrator for Content v" << ver << endl;
        cout << "Sample Program:  SDocRoutingListingICM"          << endl;
        cout << "-------------------------------------------"     << endl;
        cout << " Database: " << database                         << endl;
        cout << " UserName: " << userName                         << endl;
        cout << "===========================================\n"   << endl;

        try{
            //-------------------------------------------------------------
            // Connect to datastore
            //-------------------------------------------------------------
            // See Sample SConnectDisconnectICM for more information
            cout << "Connecting to datastore (Database '" << database << "', UserName '" << userName << "')..." << endl;

                DKDatastoreICM* dsICM = new DKDatastoreICM();  // Create new datastore object.
                dsICM->connect(database,userName,password,""); // Connect to the datastore.

            cout << "Connected to datastore (Database '" << dsICM->datastoreName() << "', UserName '" << dsICM->userName() << "')." << endl;

            //---------------------------------------------------------------------
            // Routing Service Object
            //---------------------------------------------------------------------
            // DKDocRoutingServiceICM:  Provides the core routing services such as
            //                          like starting, terminating, continuing, 
            //                          suspending, and resuming a process.
            cout << "Creating Routing Service Object..." << endl;

                DKDocRoutingServiceICM* routingService = new DKDocRoutingServiceICM(dsICM);     // Obtain the core routing service object.

            cout << "Created Routing Service & Management Objects." << endl;

            //------------------------------------------------------------------------
            // Start a Process
            //------------------------------------------------------------------------
            // As shown in SDocRoutingProcessingICM, a process may easily be created
            // as shown below.
            cout << "Starting an Instance of Process 'S_claimProcess'..." << endl;

                DKDDO* ddoFolder = dsICM->createDDO("S_simple", DK_CM_FOLDER); // Using Item Type Created in SItemTypeCreationICM.  May be replaced by any Item Type.
                ddoFolder->add();  // Save folder to the persistent datastore.

  	            DKString workPackagePidStr = routingService->startProcess("S_claimProcess", ((DKPidICM*)ddoFolder->getPidObject())->pidString(), 1, userName); // Using the UserName specified by commandline to log on in this Sample.

                delete(ddoFolder);   // Free Memory

            cout << "Started an Instance of Process 'S_claimProcess'." << endl;

            //------------------------------------------------------------------------
            // List Routing Processes
            //------------------------------------------------------------------------
            // List all running routing processes in the System.
            cout << "Listing Routing Processes in System..." << endl;
            
                listRoutingProcesses(dsICM);  // Use wrapper provided in this sample.
            
            cout << "Listed Routing Processes in System." << endl;

            //------------------------------------------------------------------------
            // List Work Nodes
            //------------------------------------------------------------------------
            // list all Work Nodes in the system.
            cout << "Listing all Work Nodes in System..." << endl;

                listWorkNodes(dsICM);  // Use wrapper provided in this sample.

            cout << "Listed all Work Nodes in System." << endl;

            //------------------------------------------------------------------------
            // List Work Lists
            //------------------------------------------------------------------------
            // list all Work Lists in the system.
            cout << "Listing all Work Lists in System..." << endl;

                listWorkLists(dsICM);  // Use wrapper provided in this sample.

            cout << "Listed all Work Lists in System." << endl;

            //------------------------------------------------------------------------
            // Count the number of Packages curreetly in a Work List
            //------------------------------------------------------------------------
            // Work Lists are managed by an administrator or process owner.
            cout << "Counting Packages Currently In Work List 'S_fillClaimWL'..." << endl;

                int count = routingService->getCount("S_fillClaimWL",userName);  // The userName specified here must be the process owner.  In this case, the process is owned by the user running the sample.
            
                cout << "  # Work Packages in Work List:  " << count << endl;

            cout << "Counted Packages Currently In Work List 'S_fillClaimWL'." << endl;

            //------------------------------------------------------------------------
            // Obtain all Work Packages in a Work List
            //------------------------------------------------------------------------
            // The process owner may obtain all work packages in the Work Nodes covered
            // by the specified Work List.
            cout << "Obtaining all Work Packages in a Work List..." << endl;
            
                long arraySize = -1;   // Size of array (specified by the OOAPI).
                DKString* workPackagePIDs = routingService->listWorkPackagePidStrings("S_fillClaimWL",userName,arraySize); // The userName specified here must be the process owner.  In this case, the process is owned by the user running the sample.

                // Print Work Package PIDs
                cout << "Work Packages in Work List:  (" << arraySize << ")" << endl;
                for(int i=0; i< arraySize; i++)
                    cout << "     - PID:  " << workPackagePIDs[i] << endl;

                delete[] workPackagePIDs;

            cout << "Obtained all Work Packages in a Work List." << endl;

            //------------------------------------------------------------------------
            // Retrieve & List Work Package Details
            //------------------------------------------------------------------------
            // The User-Application is responsible for ordering priorities if desired.
            // The Work Package may be retrieved and printed.
            //
            // In this Sample, retrieving & printing the first Work Package in the
            // Work List.
            cout << "Obtaining / Retrieving New Work Package..." << endl;
        
                DKWorkPackageICM* workPackage = routingService->retrieveWorkPackage(workPackagePidStr,false); // specifying false to not check it out.  'true' would check out the object as well.
                printWorkPackage(workPackage); // Print the Work Package using the wrapper provided in this Sample.

                delete(workPackage);           // Free Memory

            cout << "Obtained / Retrieved New Work Package." << endl;

            //------------------------------------------------------------------------
            // Terminate the process started at the beginning of this sample.
            //------------------------------------------------------------------------
            cout << "Terminating Routing Process..." << endl;

                routingService->terminateProcess(workPackagePidStr);
                
                delete(routingService);     // Free Memory

            cout << "Terminated Routing Process." << endl;

            //-------------------------------------------------------------
            // Disconnect from datastore & Destroy Reference
            //-------------------------------------------------------------
            // See Sample SConnectDisconnectICM for more information
            cout << "Disconnecting from datastore & destroying reference..." << endl;            

                dsICM->disconnect();
                delete(dsICM);

            cout << "Disconnected from datastore & destroying reference." << endl;
    
            //-------------------------------------------------------------
            // Sample program completed without exception
            //-------------------------------------------------------------
            cout << "\n==========================================" << endl;
            cout << "Sample program completed."                    << endl;
            cout << "==========================================\n" << endl;
        }
        //------------------------------------------------------------
        // Catch & Print Exceptions        
        //------------------------------------------------------------
        catch (DKException &exc){
            SConnectDisconnectICM::printException(exc);      // Print the exception to the screen
            return(SConnectDisconnectICM_RC_SAMPLE_FAILED);  // Report using return code that it failed
        }
        return(SConnectDisconnectICM_RC_SAMPLE_PASSED);      // If no exceptions were thrown, return a code to indicate passed.
    }// end main

    //=================================================================
    // Wrapper Functions
    //=================================================================
    // The following are wrapper functions for functionality covered
    // in this sample.  These functions may be used by other samples.

   /**
    * List all running Routing Processes in the System.
    * @param dsICM  Connected DKDatastoreICM object.
    **/
    void SDocRoutingListingICM::listRoutingProcesses(DKDatastoreICM* dsICM){
        
        DKDocRoutingServiceMgmtICM* routingMgmt = new DKDocRoutingServiceMgmtICM(dsICM);   // Obtain the Routing Management object.
        
        dkCollection* processes = routingMgmt->listProcesses();   // Obtain list of all routing processes running.

		if (processes && (processes->cardinality()>0)) 
		{
        
			cout << "Running Processes:  (" << processes->cardinality() << ")" << endl;
			dkIterator* iter = processes->createIterator();
			while(iter->more()){
				DKProcessICM* proc = (DKProcessICM*) iter->next()->value();    // Move pointer to next element and obtain that next element.
				cout << "     - " << proc->getName() << ":  " << proc->getDescription() << endl;
				delete(proc);                                         // Free Memory
			}
			delete(iter);                                             // Free Memory
			delete(processes);
		}
        delete(routingMgmt);
    }//end listRoutingProcesses

   /**
    * List all Work Nodes in the System.
    * @param dsICM  Connected DKDatastoreICM object.
    **/
    void SDocRoutingListingICM::listWorkNodes(DKDatastoreICM* dsICM){

        DKDocRoutingServiceMgmtICM* routingMgmt = new DKDocRoutingServiceMgmtICM(dsICM);   // Obtain the Routing Management object.

        dkCollection* workNodes = routingMgmt->listWorkNodes();        // Obtain all Work Nodes in the System.
       
	    if (workNodes &&  ( workNodes->cardinality()>0) ) 
		{
			cout << "Work Nodes in System:  (" << workNodes->cardinality() << ")" << endl;
			dkIterator* iter = workNodes->createIterator();
			while(iter->more()){
				DKWorkNodeICM* workNode = (DKWorkNodeICM*) iter->next()->value();  // Move pointer to next element and obtain that next element.   
				if(workNode->getType()==0)
					cout << "       Normal Node - " << workNode->getName() << ":  " << workNode->getDescription() << endl;
				else
					cout << "     Collection Pt - " << workNode->getName() << ":  " << workNode->getDescription() << endl;
				delete(workNode);                                         // Free Memory
			}
			delete(iter);                                                 // Free Memory
			delete(workNodes);
		}
        delete(routingMgmt);
    }//end listWorkNodes

   /**
    * List all Work Lists in the System.
    * @param dsICM  Connected DKDatastoreICM object.
    **/
    void SDocRoutingListingICM::listWorkLists(DKDatastoreICM* dsICM){

        DKDocRoutingServiceMgmtICM* routingMgmt = new DKDocRoutingServiceMgmtICM(dsICM);   // Obtain the Routing Management object.

        dkCollection* workLists = routingMgmt->listWorkLists();      // Obtain all Work Lists in the System.

		if (workLists && (workLists->cardinality()>0) )
		{

			cout << "Work Lists in System:  (" << workLists->cardinality() << ")" << endl;
			dkIterator* iter = workLists->createIterator();
			while(iter->more()){
				DKWorkListICM* workList = (DKWorkListICM*) iter->next()->value();  // Move pointer to next element and obtain that next element.   
				cout << "     - " << workList->getName() << ":  " << workList->getDescription() << endl;
				delete(workList);                                         // Free Memory
			}
			delete(iter);                                                 // Free Memory
			delete(workLists);
		}
        delete(routingMgmt);
        
    }//end listWorkLists

   /**
    * Print all Work Package PIDs in a Work List
    * @param dsICM         Connected DKDatastoreICM object.
    * @param workListName  Name of the Work List.
    * @param processOwner  Owner of the process.
    **/
    void SDocRoutingListingICM::printWorkPackagePidsFromWorkList(DKDatastoreICM* dsICM, char* workListName, char* processOwner){
    
        DKDocRoutingServiceICM* routingService = new DKDocRoutingServiceICM(dsICM);                   // Obtain the Routing Management object.
            
        long arraySize = -1;  // Size to be set by the API.
        DKString* workPackagePIDs = routingService->listWorkPackagePidStrings("workListName",processOwner,arraySize); // The userName specified here must be the process owner.  In this case, the process is owned by the user running the sample.

        // Print Work Package PIDs
        cout << "Work Packages in Work List:  (" << arraySize << ")" << endl;
        for(int i=0; i< arraySize; i++)
            cout << "     - PID:  " << workPackagePIDs[i] << endl;
    
        delete[] workPackagePIDs;   // Free Memory
        delete(routingService);

    }//end printWorkPackagePidsFromWorkList

   /**
    * Print the specified Work Package.
    * @param workPackage  The Work Package to be Printed.
    **/
    void SDocRoutingListingICM::printWorkPackage(DKWorkPackageICM* workPackage){

        cout << "---------------------------------------------"              << endl;
        cout << "                Work Package"                               << endl;
        cout << "---------------------------------------------"              << endl;
        cout << "    Process Name:  " << workPackage->getProcessName()       << endl;
        cout << "  work Node Name:  " << workPackage->getWorkNodeName()      << endl;
        cout << "           Owner:  " << workPackage->getOwner()             << endl;
        cout << "        Priority:  " << workPackage->getPriority()          << endl;
        cout << " User Last Moved:  " << workPackage->getUserLastMoved()     << endl;
        cout << " Time Last Moved:  " << workPackage->getTimeLastMoved()     << endl;
        cout << "   Suspend State:  " << workPackage->getSuspendState()      << endl;
        cout << "    Notify State:  " << workPackage->getNotifyState()       << endl;
        cout << "     Notify Time:  " << workPackage->getNotifyTime()        << endl;
        cout << "     Resume Time:  " << workPackage->getResumeTime()        << endl;
        cout << "Work Package Pid:  " << workPackage->getPidString()         << endl;
        cout << "        Item Pid:  " << workPackage->getItemPidString()     << endl;

        // Print any requirements needed to move onto next node
        dkCollection* requirements = workPackage->getResumeList();

		if (requirements && (requirements->cardinality()>0) )
		{
			dkIterator* iter = requirements->createIterator();
			cout << "     Resume List:  '" << requirements->cardinality() << "' Requirements:" << endl;
			while (iter->more()){
				DKResumeListEntryICM* resumeRequirement = (DKResumeListEntryICM*) iter->next()->value();    // Move pointer to next element and obtain that next element.
				cout << "          Required Item Type:  " << resumeRequirement->getRequiredItemTypeName() << endl;
				cout << "              Quanity Needed:  " << resumeRequirement->getQuantityNeeded() << "\n" << endl;
			}
			delete(iter);      // Free Memory
		}
        
    }//end printWorkPackage()
