// ----------------------------------------------------------------------------
// @copyright(disclaimer) 
// Licensed Materials - Property of IBM
// IBM  Enterprise Information Portal  for Multiplatforms V8.1  (program number 5724-B43)
// (c ) Copyright IBM Corp. 1998,  2002. All Rights Reserved.
// 
// US Government Users Restricted Rights                                  
// Use, duplication or disclosure restricted by GSA ADP Schedule          
// Contract with IBM Corp.     
// 
// DISCLAIMER OF WARRANTIES :                                             
//                                                                        
// Permission is granted to copy and modify this  Sample code, and to
// distribute modified versions provided that both the copyright        notice,-
// and this permission notice and warranty disclaimer  appear in all copies and
// modified  versions. 
// 
// THIS SAMPLE CODE IS LICENSED TO YOU AS-IS. IBM  AND ITS SUPPLIERS AND LICENSORS
// DISCLAIM ALL WARRANTIES, EITHER EXPRESS OR IMPLIED, IN SUCH SAMPLE CODE,
// INCLUDING THE WARRANTY OF NON-INFRINGEMENT AND THE IMPLIED WARRANTIES OF
// MERCHANTABILITY OR FITNESS FOR A PARTICULAR PURPOSE. IN NO EVENT WILL
// IBM OR ITS LICENSORS OR SUPPLIERS BE LIABLE FOR ANY DAMAGES ARISING OUT
// OF THE USE OF  OR INABILITY TO USE THE SAMPLE CODE, DISTRIBUTION OF THE
// SAMPLE CODE, OR COMBINATION OF THE SAMPLE CODE WITH ANY OTHER CODE. IN
// NO EVENT SHALL IBM OR ITS LICENSORS AND SUPPLIERS BE LIABLE FOR ANY LOST
// REVENUE, LOST PROFITS OR DATA, OR FOR DIRECT, INDIRECT, SPECIAL,
// CONSEQUENTIAL,INCIDENTAL OR PUNITIVE DAMAGES, HOWEVER CAUSED AND
// REGARDLESS OF THE THEORY OF LIABILITY,-, EVEN IF IBM OR ITS LICENSORS OR
// SUPPLIERS HAVE BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGES.                   
// @endCopyright       
// ----------------------------------------------------------------------------

/*
 * Filename       : TExecutedCQExprWithCallbackDD.cpp
 * Description    : Run execute with callback and pass in parametric query string. Then
 *                  display the DDO information
 *
 */

#include "stdafx.h"
#include "TExecuteCQExprWithCallbackDD.h"

#include "DKDatastoreDD.hpp"
#include "DKOpCode.hpp"
#include "DKCQExpr.hpp"
#include "DKQSubExpr.hpp"
#include "DKQExpr.hpp"
#include "DKQTerm.hpp"
#include "dkQueryableCollection.hpp"
#include "dkResultSetCursor.hpp"
#include "DKAny.hpp"
#include "DKResults.hpp"
#include "DKEntityDefDD.hpp"
#include "DKAttrDefDD.hpp"
#include "DKException.hpp"
#include "TCallbackDD.hpp"

/////////////////////////////////////////////////////////////////////////////
// The one and only application object

CWinApp theApp;

void printResults(dkResultSetCursor* results)
{
	cout << "Results" << endl;
	cout << "Cardinality: " << results->cardinality() << endl;
}

int _tmain(int argc, TCHAR* argv[], TCHAR* envp[])
{
	int nRetCode = 0;

	// initialize MFC and print and error on failure
	if (!AfxWinInit(::GetModuleHandle(NULL), NULL, ::GetCommandLine(), 0))
	{
		// TODO: change error code to suit your needs
		cerr << _T("Fatal Error: MFC initialization failed") << endl;
		nRetCode = 1;
	}
	for( int nbrArgs = 1; nbrArgs < argc; nbrArgs++ )
	{
		cout << "TListAttrsDD parm " << nbrArgs << "= "<< argv[nbrArgs] << endl;
	}
	if( nbrArgs < 4 )
	{
	    cout << "TListAttrsDD usage = <serverName> <userId> <pw>" << endl;
		return -1;
	}

	DKDatastoreDD* dd;
	try
	{
		DKString server = argv[1];
		DKString uid = argv[2];
		DKString pw = argv[3];
		dd = new DKDatastoreDD();
		dd->connect(server, uid, pw,"");
		if( dd->isConnected() )
		{
			/**** Query 1 ****/
			DKString strTmp;
		    TCallbackDD cbkDD;
			DKAny any;
			long arraySize = 1;
			DKNVPair soptions[1];
			DKNVPair qoptions[1];
			DKCQExpr* cqe = new DKCQExpr();
			DKQExpr* qe = new DKQExpr();
			DKQTerm* qterm = new DKQTerm("DocType",DK_CM_OPCODE_LIKE,"D*");
			DKQSubExpr* sqe = new DKQSubExpr();
			cqe->setQLType(DK_CM_PARAMETRIC_QL_TYPE);
			qe->setQLType(DK_CM_PARAMETRIC_QL_TYPE);
			sqe->setTranslation(false);
			sqe->setEntityName("WebPublishing");
			sqe->setQueryTerm(qterm);
			strTmp = "5";
			any = strTmp;
			soptions[0].setName("MAX_RESULTS");
			soptions[0].setValue(any);
			qe->addSubQuery(sqe);
			strTmp = "NO";
			any = strTmp;
			qoptions[0].setName("CONTENT");
			qoptions[0].setValue(any);
			cqe->setQueryExpr(qe);

			dd->executeWithCallback(cqe,(dkCallback*)&cbkDD);
			cout << "query executed" << endl;
			
			DKBoolean isDone = FALSE;
			while (isDone == FALSE)
			{
				cout << "press <enter> to continue... " << endl;
				getchar();
				isDone = cbkDD.isDone();
				cout << "isDone: " << ((isDone == TRUE) ? "True" : "False") << endl;
			}

			delete cqe;
			delete qe;
			delete qterm;
			delete sqe;
			cout << "datastore disconnected" << endl;
			dd->disconnect();
		}
	}
	catch( DKException dke )
	{
		for( long i = 0; i < dke.textCount(); i++ )
		{
			const char* text = dke.text(i);
			if( text != 0 )
				cout << text << endl;
		}
	}
	catch( ... )
	{
		cout << "TExecuteCQExprWithCallbackDD FAILURE! Caught unexpected exception." << endl;
	}


	delete dd;
	return nRetCode;
}
