/*
@copyright(disclaimer)

Licensed Materials - Property of IBM
IBM  Enterprise Information Portal  for Multiplatforms V8.1  (program number 5724-B43)
(c) Copyright IBM Corp. 1998,  2002. All Rights Reserved.

US Government Users Restricted Rights
Use, duplication or disclosure restricted by GSA ADP Schedule
Contract with IBM Corp.

DISCLAIMER OF WARRANTIES :

Permission is granted to copy and modify this  Sample code, and to
distribute modified versions provided that both the copyright
notice,- and this permission notice and warranty disclaimer  appear in all copies and modified
versions.

THIS SAMPLE CODE IS LICENSED TO YOU AS-IS.
IBM  AND ITS SUPPLIERS AND LICENSORS  DISCLAIM
ALL WARRANTIES, EITHER EXPRESS OR IMPLIED, IN SUCH SAMPLE CODE, INCLUDING THE WARRANTY
OF NON-INFRINGEMENT AND THE IMPLIED WARRANTIES OF MERCHANTABILITY OR FITNESS FOR A
PARTICULAR PURPOSE. IN NO EVENT WILL IBM OR ITS LICENSORS OR SUPPLIERS BE LIABLE FOR ANY
DAMAGES ARISING OUT OF THE USE OF  OR INABILITY TO USE THE SAMPLE CODE, DISTRIBUTION OF
THE SAMPLE CODE, OR COMBINATION OF THE SAMPLE CODE WITH ANY OTHER CODE. IN NO EVENT SHALL
IBM OR ITS LICENSORS AND SUPPLIERS BE LIABLE FOR ANY LOST REVENUE, LOST PROFITS OR DATA,
OR FOR DIRECT, INDIRECT, SPECIAL, CONSEQUENTIAL,INCIDENTAL OR PUNITIVE DAMAGES, HOWEVER
CAUSED AND REGARDLESS OF THE THEORY OF LIABILITY,-, EVEN IF IBM OR ITS LICENSORS OR SUPPLIERS
HAVE BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGES.

@endCopyright
*/

import com.ibm.mm.beans.*;

/**
 * Sample to add a text annotation to an item.  For OnDemand, additional arguments
 * can specify the page, x, y coordinates.
 * <p>
 * Note: The annotation added by this sample is not compatible with the CM viewer.  Please
 * use the annotation services (com/ibm/mm/viewer/annotation) for support in manipulating
 * annotations compatible with the CM viewer.
 */
public class TAddAnnotation {

	public static void main(String[] args) throws Exception {
		if (args.length < 6) {
			System.out.print(
				"Adds a text annotation to a document. \n"
					+ "Syntax: \n"
					+ "  java TAddAnnotation <dstype> <server> <userid> <password> <entity> <condition> / <annotationText> [/ <page> <x> <y>]\n"
					+ "where: \n"
					+ "  <condition> is a search condition. \n"
					+ "  <attrs> are name=value syntax specifying the attributes. \n"
					+ "The condition and attributes are separated with a slash. \n"
					+ "Examples: \n"
					+ "  java TAddAnnotation OD odserver user1 pass 'Telephone Statements' CustName like '%DAGWOOD%' / A new annotation. \n");
			return;
		}

		// Read args
        String dstype = args[0];
        String server = args[1];
        String userid = args[2];
        String password = args[3];
        String entity = args[4];
        String condition = "";
        int i;
        for (i = 5; i < args.length && !args[i].equals("/"); i++) {
            condition += " " + args[i];
        }
        i++;
        String annotation = "";
        for (; i < args.length && !args[i].equals("/"); i++) {
            annotation += " " + args[i];
        }
        int page = 1;
        if (i < args.length) {
        	page = Integer.valueOf(args[i]).intValue();
        	i++;
        }
        int x = 0;
        if (i < args.length) {
        	x = Integer.valueOf(args[i]).intValue();
        	i++;
        }
        int y = 0;
        if (i < args.length) {
        	y = Integer.valueOf(args[i]).intValue();
        	i++;
        }

        // Create beans.  The CMBSearchResults, CMBDataManagement, and
        // CMBQueryService beans can be obtained from the CMBConnection bean.
        CMBConnection connection = new CMBConnection();
        CMBQueryService queryService = connection.getQueryService();
        CMBSearchResults searchResults = new CMBSearchResults();
        searchResults.setConnection(connection);
        CMBDataManagement dataManagement = connection.getDataManagement();

        // Connect (using method from TConnect sample
        TConnect.connect(connection, dstype, server, userid, password);

		// Search for items to annotate
        TSearch.search(queryService, dstype, entity, condition, false);
		searchResults.newResults(queryService.getResults());
		for (i = 0; i < searchResults.getCount(); i++) {
			CMBItem item = searchResults.getItem(i);

			// Display the current annotations on the item
			dataManagement.setDataObject(item);
			CMBAnnotation[] annotations = dataManagement.getAnnotations();
			System.out.println("   Item's annotations:");
			for (int j = 0; j < annotations.length; j++) {
				CMBAnnotation annot = annotations[j];
				System.out.println("    " + annot.getStringData());
			}

			// Add an annotation
			System.out.println("  Adding annotation.");
            dataManagement.checkOut();
			addAnnotation(dataManagement, item, page, true, true, "123", "abc", (short)x, (short)y, 'B', annotation);
            dataManagement.checkIn();

			// Display the new annotations on the item
			dataManagement.setDataObject(item);
			annotations = dataManagement.getAnnotations();
                //only OD server supports getStringData
                //for other servers, use TView to view the annotaion.
	if (dstype.equals("OD")) {
			System.out.println("   Item's annotation's are now: ");
			for (int j = 0; j < annotations.length; j++) {
				CMBAnnotation annot = annotations[j];
				System.out.println("    " + annot.getStringData());
			}
	}
		}

		System.out.println("Disconnecting from server");
		connection.disconnect();

	}

	/**
	 * Adds an annotation to an item.  Many of the parameters are OnDemand only
	 * @param dataManagement an instance of CMBDataManagement bean
	 * @param item an instance of CMBItem for the document
	 * @param pageNumber number of page to add annotation
	 * @param isPublic annotation is public access
	 * @param okToCopy annotation is okay to copy
	 * @param groupName group name
	 * @param x x coordinate of annotation on page
	 * @param y y coordinate of annotation on page
	 * @param color color of annotation
	 * @param text text for annotation
	 */
	public static void addAnnotation(CMBDataManagement dataManagement, CMBItem item,
		int pageNumber,	boolean isPublic, boolean okToCopy, String timeStamp,
		String groupName, short x, short y, char color, String text) 
		throws CMBException {

		CMBAnnotation annotation = new CMBAnnotation(text, item.getServerType());
		annotation.setPageNumber(pageNumber);
		annotation.setX(x);
		annotation.setY(y);
		if (item.getServerType().equals(CMBBaseConstant.CMB_DSTYPE_OD)) {
			annotation.setPublic(isPublic);
			annotation.setOkToCopy(okToCopy);
			annotation.setTimestamp(timeStamp);
			annotation.setGroupName(groupName);
			annotation.setColor(color);
		}
        annotation.setMimeType("text/plain"); //required
		dataManagement.addAnnotation(annotation);
	}

}