/*
@copyright(disclaimer)

Licensed Materials - Property of IBM
IBM  Enterprise Information Portal  for Multiplatforms V8.1  (program number 5724-B43)
(c) Copyright IBM Corp. 1998,  2002. All Rights Reserved.

US Government Users Restricted Rights
Use, duplication or disclosure restricted by GSA ADP Schedule
Contract with IBM Corp.

DISCLAIMER OF WARRANTIES :

Permission is granted to copy and modify this  Sample code, and to
distribute modified versions provided that both the copyright
notice,- and this permission notice and warranty disclaimer  appear in all copies and modified
versions.

THIS SAMPLE CODE IS LICENSED TO YOU AS-IS.
IBM  AND ITS SUPPLIERS AND LICENSORS  DISCLAIM
ALL WARRANTIES, EITHER EXPRESS OR IMPLIED, IN SUCH SAMPLE CODE, INCLUDING THE WARRANTY
OF NON-INFRINGEMENT AND THE IMPLIED WARRANTIES OF MERCHANTABILITY OR FITNESS FOR A
PARTICULAR PURPOSE. IN NO EVENT WILL IBM OR ITS LICENSORS OR SUPPLIERS BE LIABLE FOR ANY
DAMAGES ARISING OUT OF THE USE OF  OR INABILITY TO USE THE SAMPLE CODE, DISTRIBUTION OF
THE SAMPLE CODE, OR COMBINATION OF THE SAMPLE CODE WITH ANY OTHER CODE. IN NO EVENT SHALL
IBM OR ITS LICENSORS AND SUPPLIERS BE LIABLE FOR ANY LOST REVENUE, LOST PROFITS OR DATA,
OR FOR DIRECT, INDIRECT, SPECIAL, CONSEQUENTIAL,INCIDENTAL OR PUNITIVE DAMAGES, HOWEVER
CAUSED AND REGARDLESS OF THE THEORY OF LIABILITY,-, EVEN IF IBM OR ITS LICENSORS OR SUPPLIERS
HAVE BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGES.

@endCopyright
*/

import  com.ibm.mm.beans.*;

/**
 * Unlock an item or items.
 */
public class TUnlock {

	public static void main(String args[]) throws Exception {
        if (args.length < 5) {
        	System.out.println(
                "Unlocks all items matching search pattern. \n"
                    + "Syntax: \n"
                    + "  java TUnlock <dstype> <server> <userid> <password> <entity> <condition> \n"
                    + "  <dstype> is the type of server (Fed, ICM, DL, OD, DB2) \n"
                    + "  <server> is the name of the server or database.  This can be \n"
                    + "           followed by a connect string in parenthesis. \n"
                    + "  <userid> is the userid \n"
                    + "  <password> is the password \n"
                    + "  <entity> the name of the entity (aka item type, table) to search. \n"
                    + "  <condition> the condition portion of the query string.  The syntax \n"
                    + "              of the condition varies depending on server type. \n"
                    + "Examples: \n"
                    + "  java TUnlock DL libsrvrn frnadmin password NOINDEX Userid LIKE 'user1' \n"
                    + "  java TUnlock Fed cmbdb cmbadmin password NoIndex Userid LIKE 'user1' \n"
                    + "  java TUnlock DB2 sample db2admin password EMP_PHOTO PHOTO_FORMAT LIKE 'gif' \n"
                    + "  java TUnlock ICM icmnlsdb icmadmin password Book @Title LIKE \\\"The Java Book\\\"");
            System.exit(0);
        }

		// Read args
        String dstype = args[0];
        String server = args[1];
        String userid = args[2];
        String password = args[3];
        String entity = args[4];
        String condition = "";
        for (int i = 5; i < args.length; i++) {
            condition += " " + args[i];
        }

        // Create beans.  The CMBSearchResults, CMBDataManagement, and
        // CMBQueryService beans can be obtained from the CMBConnection bean.
        CMBConnection connection = new CMBConnection();
        CMBQueryService queryService = connection.getQueryService();
        CMBSearchResults searchResults = new CMBSearchResults();
        searchResults.setConnection(connection);
        CMBDataManagement dataManagement = connection.getDataManagement();

        // Connect (using method from TConnect sample
        TConnect.connect(connection, dstype, server, userid, password);

        // Search to find items to unlock
        TSearch.search(queryService, dstype, entity, condition, false);
        searchResults.newResults(queryService.getResults());
        System.out.println("There are " + searchResults.getCount() + " items found.");

        // Unlock the items found
        if (searchResults.getCount() == 0) {
            System.out.println("No items unlocked");
        } else {
            for (int i = 0; i < searchResults.getCount(); i++) {
                CMBItem item = searchResults.getItem(i);
                unlock(dataManagement, item);
            }
        }

        // Disconnect (using TConnect sample)
        TConnect.disconnect(connection);

    }

    /**
     * Unlocks an item.
     * @param dataManagement an instance of CMBDataManagement bean.
     * @param item an instance of CMBItem for the item to unlock
     */
    public static void unlock(CMBDataManagement dataManagement, CMBItem item) throws CMBException {
        System.out.println("    Unlocking item "+item.getName());
        dataManagement.setDataObject(item);
        try {
	        dataManagement.unlock();
        } catch (CMBMethodNotSupportedException e) { // server doesn't support unlock
	        dataManagement.checkIn();				 // use checkin instead
	    }
    }

}
