/*
 * @copyright(disclaimer) 
 * Licensed Materials - Property of IBM
 * IBM  Enterprise Information Portal  for Multiplatforms V8.1  (program number 5724-B43)
 * (c ) Copyright IBM Corp. 1998,  2002. All Rights Reserved.
 * 
 * US Government Users Restricted Rights                                  
 *  Use, duplication or disclosure restricted by GSA ADP Schedule          
 *  Contract with IBM Corp.     
 * 
 * DISCLAIMER OF WARRANTIES :                                             
 *                                                                        
 * Permission is granted to copy and modify this  Sample code, and to           
 * distribute modified versions provided that both the copyright        
 * notice,- and this permission notice and warranty disclaimer  appear in all copies and modified  
 * versions. 
 * 
 * THIS SAMPLE CODE IS LICENSED TO YOU AS-IS. 
 * IBM  AND ITS SUPPLIERS AND LICENSORS  DISCLAIM     
 * ALL WARRANTIES, EITHER EXPRESS OR IMPLIED, IN 
 * SUCH SAMPLE CODE, INCLUDING THE WARRANTY OF 
 * NON-INFRINGEMENT AND THE IMPLIED WARRANTIES 
 * OF MERCHANTABILITY OR FITNESS FOR A PARTICULAR 
 * PURPOSE. IN NO EVENT WILL IBM OR ITS LICENSORS OR
 * SUPPLIERS BE LIABLE FOR ANY DAMAGES ARISING OUT 
 * OF THE USE OF  OR INABILITY TO USE THE SAMPLE CODE, 
 * DISTRIBUTION OF THE SAMPLE CODE, OR COMBINATION 
 * OF THE SAMPLE CODE WITH ANY OTHER CODE. IN NO 
 * EVENT SHALL IBM OR ITS LICENSORS AND SUPPLIERS BE 
 * LIABLE FOR ANY LOST REVENUE, LOST PROFITS OR DATA, 
 * OR FOR DIRECT, INDIRECT, SPECIAL, CONSEQUENTIAL,
 * INCIDENTAL OR PUNITIVE DAMAGES, HOWEVER CAUSED 
 * AND REGARDLESS OF THE THEORY OF LIABILITY,-, EVEN IF 
 * IBM OR ITS LICENSORS OR SUPPLIERS HAVE BEEN ADVISED 
 * OF THE POSSIBILITY OF SUCH DAMAGES.                                     
 *                                                                  
 * @endCopyright
 */

import java.io.*;
import javax.swing.*;
import java.awt.*;
import java.awt.font.*;
import java.awt.event.*;
import java.net.URL;
import java.text.*;
import java.util.*;
import com.ibm.mm.beans.gui.*;
import com.ibm.mm.beans.*;

/**
 * Adds components to content pane
 */
public class ClientPane extends Object
     implements
          CMBEditCompletedListener,
          CMBEditCancelledListener,
          CMBLogonCompletedListener , 
          CMBLogonCancelledListener,
          CMBEditItemAttributesListener,
          CMBViewVersionsListener,
          CMBViewVersionsCancelledListener,
          ActionListener { 

   // Do not remove - needed for copyright protection
   public final static String IBM_LONG_COPYRIGHT = ClientConst.IBM_LONG_COPYRIGHT;
    // CUSTOMIZE FOR YOUR INSTALL
   static transient String server="icmnlsdb";
   static  transient String helpURLString="file:///c:/Cmbroot/Samples/java/beans/gui/ClientHelp.html";  // dir where help file is found
   //static  transient String helpURLString="file:///doc/ClientHelp.html";  // dir where help file is found
   static  transient short connectionType=CMBBaseConstant.CMB_CONNTYPE_DYNAMIC;
   static  transient int maxresults=-1;  // -1 indicates no maximum


   public final static String PRODUCT_NAME = ResourceMgr.getString("CLIENT_TITLEBAR_TITLE");
   public final static String UPDATE_ATTRIBUTES = ResourceMgr.getString("UPDATE_ITEM_ATTRIBUTES");

   private boolean urlFlag=true;
   private static ClientInterface client;  // application vs applet-specific methods
   
   // Beans getters 
   private transient CMBSearchResultsViewer searchResultsViewer=new CMBSearchResultsViewer();
   public CMBSearchResultsViewer getSearchResultsViewerBean() {
    return searchResultsViewer;
   }
   private transient CMBSearchTemplateViewer searchTemplateViewer=new CMBSearchTemplateViewer();
   public CMBSearchTemplateViewer getSearchTemplateViewerBean() {
    return searchTemplateViewer;
   }
   private transient CMBSearchPanel searchPanel=new CMBSearchPanel();
   public CMBSearchPanel getSearchPanelBean() {
    return searchPanel;
   }
   private transient CMBSearchTemplateList searchTemplateList=new CMBSearchTemplateList();
   public CMBSearchTemplateList getSearchTemplateListBean() {
    return searchTemplateList;
   }
   private transient CMBDocumentViewer  documentViewer = new CMBDocumentViewer();
   public CMBDocumentViewer getDocumentViewerBean() {
    return documentViewer;
   }
   private transient CMBItemAttributesEditor itemAttributesEditor = new CMBItemAttributesEditor();
   public CMBItemAttributesEditor getItemAttributesEditorBean() {
    return itemAttributesEditor;
   }
   protected transient CMBLogonPanel  logonPanel = new CMBLogonPanel();
   public CMBLogonPanel getLogonPanelBean() {
    return logonPanel;
   }
   protected transient CMBVersionsViewer  versionsViewer = new CMBVersionsViewer();
   public CMBVersionsViewer getVersionsViewerBean() {
    return versionsViewer;
   }
   private transient CMBConnection connection=new CMBConnection();
   public CMBConnection getConnectionBean() {
    return connection;
   }
   
   
   
    // Get the orientation of the current language for BiDi purposes.
    public static boolean isLeftToRight() {
        return ComponentOrientation.getOrientation(Locale.getDefault()).isLeftToRight();
    }
   
   // Used for BiDi layout calculations
   private boolean ltr = isLeftToRight();
 
   private transient FunctionBar functionBar=null;
   private transient LogonDialog logonDialog=null;
   private transient JDialog attributesEditorDialog=null;
   private transient JDialog versionsViewerDialog=null;
   private transient JSplitPane split2= null;
   private transient JPanel searchArea = null;
   private transient JLabel statusBar = new JLabel("               ");
   
   // Getter to allow dialogs to set messages on the status bar
   public JLabel getStatusBar() {
    return statusBar;
   }
   
   private JPanel bottomPane=null;
   // Set to either the JFrame or the JApplet depending on the way we're started
   private transient Container mainWindow;
   public JFrame getAppFrame() {
        if (client instanceof JFrame) 
            return (JFrame)client;
        return null;
   }   
   

  
   // Useful for testing different locales
   void setLocale( Locale l) {
        searchResultsViewer.setLocale(l);
        searchTemplateViewer.setLocale(l);
        searchPanel.setLocale(l);
        searchTemplateList.setLocale(l);
        logonPanel.setLocale(l);
        logonDialog.setLocale(l);
        attributesEditorDialog.setLocale(l);
        mainPanel.setLocale(l);
   }


   private transient JPanel mainPanel=new JPanel(new BorderLayout()) {
    public void setBounds(int xpos, int ypos, int width, int height) {
        super.setBounds(xpos,ypos,width,height);
        // Set initial position of splitter
        if (split2 != null) {
            if (ltr)
                split2.setDividerLocation(searchTemplateList.getPreferredSize().width);
            else        
                split2.setDividerLocation(width - searchTemplateList.getPreferredSize().width);
        }
    }
    
    public void setSize(int width, int height) {
        super.setSize(width, height);
        // Set initial position of splitter
        if (split2 != null) {
            if (ltr)
                split2.setDividerLocation(searchTemplateList.getPreferredSize().width);  
            else        
                split2.setDividerLocation(width - searchTemplateList.getPreferredSize().width);
        }
    }
   };
    
   
   /*
    * Constructor
    * @param contentPane - the main frame's content pane
    * @param client - the client interface object
    */
   ClientPane(Container contentPane, ClientInterface client) {
      super();
      this.mainWindow = contentPane;
      this.client = client;
      mainWindow.setLayout(new BorderLayout());
      showProperties();
      //setLookAndFeel(this);
      int rc=0;
      rc=setup();
      if (rc<0) {  // error when trying to logon,  exit application 
         this.exitProgram();
      }
      else {
        mainLayout(); // layout the main panel views
        try {
        mainWindow.add( mainPanel);
      } catch(Exception e) {
         System.out.println(" Error initializing Client Application");
        }
        functionBar.verifyButtonStates(); // gray out buttons 
        logon();
        functionBar.verifyButtonStates(); // update buttons after logon
        //split2.resetToPreferredSizes(); // to reflect width of search template names
     } // end else 
   }


    private int setup() {
        try {
            // Set special connection properties here
            //For IP:
            //connection.setConnectString("APPL=01;FAFIP=9.67.43.83;IODMIP=9.67.43.83;FAFPORT=3061;IODMPORT=3082;FAFPROT=4000;IODMPROT=4000;FAFSITE=cs61");
            //
	        //For JDBC:
	        //connection.setConfigString("JDBCDRIVER=(COM.ibm.db2.jdbc.app.DB2Driver)");
	        // set datastore type AFTER setting the config string
	        //connection.setDsType(CMBBaseConstant.CMB_DSTYPE_JDBC);
	        //
	        //For DES:
            //connection.setConnectString("PORT=80;DESAPPID=appid;DESAPPPW=apppw");
            connection.setConnectToWorkFlow(false);// do not attempt to connect to a workflow server
            connection.setConnectionType(connectionType);
            connection.setServerName(server);
            
            // set connection property for each bean 
            // Delay init of STL, STV, SP until we determine if 
            // we will use templates or entities
            //searchTemplateViewer.setConnection(connection);
            //searchTemplateList.setConnection(connection);
            //searchPanel.setConnection(connection);
            itemAttributesEditor.setConnection(connection);
            logonPanel.setConnection(connection);
            documentViewer.setConnection(connection);
            searchResultsViewer.setConnection(connection);
            versionsViewer.setConnection(connection);
            
            // Filter out DomDoc and DES from list.  To support, you need to be able
            // to generate the proper syntax and set on the CMBQueryService bean to
            // execute a search.
            String[] serverTypes = logonPanel.getDisplayServerTypes();
            Vector filteredTypes = new Vector();
            for (int i=0, index=0; i < serverTypes.length; ++i) {
                if (!serverTypes[i].equals(CMBBaseConstant.CMB_DSTYPE_DOMDOC) &&
                    !serverTypes[i].equals(CMBBaseConstant.CMB_DSTYPE_DES)) {
                        filteredTypes.add(serverTypes[i]);
                }
            }
            
            String[] serverTypes2= new String[filteredTypes.size()];
            filteredTypes.copyInto(serverTypes2);
            logonPanel.setDisplayServerTypes(serverTypes2);


        }
            catch (Exception e) {
            Object [] args= {server};
            MessageBox mb = new MessageBox(ResourceMgr.getString("TITLEBAR_TITLE"), 50, this);
            mb.setMessageText(ResourceMgr.getString("CONNECTION_FAILURE",args));
            mb.addChoice(ResourceMgr.getString("OK"),"OK");
            mb.setSynchronousMode(true);
            mb.display();
            e.printStackTrace();
            return -1;
        }
        Font f=new Font("SansSerif",java.awt.Font.BOLD,12);
        searchTemplateList.setFont(f);
        searchResultsViewer.setDetailsHorizontalLinesVisible(false);
        searchResultsViewer.setDetailsVerticalLinesVisible(false);
        
        // Uncomment the following code to set mime2app properties on the document viewer
        // Properties mime2app = new Properties();
        
        // Programmers can add their own definitions for MIME type and application pairs
        // following these rules:
        //  key: MimeType
        //  value: a=application x=file extention
        //  a="application name"
        //  x="common file extention for this application"
        //      examples:
        //      mime2app.put("text/plain", "a=notepad x=.txt");
        //      mime2app.put("text/richtext", "a=c:/progra~1/acceso~1/wordpad x=.txt");
        //      mime2app.put("image/tiff", "a=c:\\progra~1\\acceso~1\\imagevue\\wangimg x=.tiff");
        //
        // Uncomment the next line to specify the sample viewer as the default viewer for all mimetypes
        // mime2app.put("*", "SampleViewer.class");
        
        // documentViewer.setMime2App(mime2app);
              
        return 0;
    }  

    public void actionPerformed (ActionEvent evt ) {
        if (evt.getActionCommand().equals("OK")) {
            client.exitProgram();
        }
    }

// Note:  All wacky things with panels & layouts are done to get a good 
// division of space.  Seemingly senseless extra panels are there to 
// ensure a good look 
   private void mainLayout() {
        mainPanel.removeAll();
      
      // set IBM EIP bean properties
      searchTemplateViewer.setStartSearchButtonVisible(false);
      searchTemplateViewer.setStopSearchButtonVisible(false);
      searchPanel.setSearchButtonVisible(false);
      searchPanel.setCancelButtonVisible(false);
      documentViewer.setVisible(false);
      searchResultsViewer.setExploreOnViewFolder(true);
      searchResultsViewer.setDefaultPopupEnabled(true);
      searchResultsViewer.setMultiSelectEnabled(true);
      itemAttributesEditor.setHelpButtonVisible(false);
      logonPanel.setHelpButtonVisible(false);
      versionsViewer.setHelpButtonVisible(false);
      
            
      // Create inner panels for layout
      JPanel topPane        = new JPanel();
      JPanel bottomPane     = new JPanel();
      //JPanel topPaneCenter  = new JPanel();
      searchArea            = new JPanel();
      topPane.setLayout( new BorderLayout());
      bottomPane.setLayout( new BorderLayout());
      searchArea.setLayout(new CardLayout());
  
      // Create the splitter separating the top & bottom halves of client pane
      JSplitPane split = new JSplitPane( JSplitPane.VERTICAL_SPLIT, topPane, bottomPane);
      split.setPreferredSize( new Dimension( 400, 10));
      mainPanel.add( split, BorderLayout.CENTER);
  
      // Add the status bar
      bottomPane.add( BorderLayout.SOUTH, statusBar);
	  // This is a workaround for Swing 1.2.2
	  // It does not properly set Horizontal alignment for BiDi languages
	  // Take this out when Swing is working
	  if (!ltr) {
	     statusBar.setHorizontalAlignment(JTextField.RIGHT);
      }
      
      functionBar=new FunctionBar(client,this);
      
      // Add JToolBar to the north of the main panel to
      // ensure proper docking function.
      mainPanel.add(functionBar, BorderLayout.NORTH);
  
    // Add the search template viewer card
    if (searchTemplateList.getStyle() == CMBSearchTemplateList.DROP_DOWN) {
        JPanel p2 = new JPanel();
        p2.setLayout(new BorderLayout(0,0));
        JPanel panel1 = new JPanel();
        panel1.setLayout(new BorderLayout());
        if (ltr)
            panel1.add(searchTemplateList, BorderLayout.WEST);
        else 
            panel1.add(searchTemplateList, BorderLayout.EAST);
        panel1.add(new JPanel(), BorderLayout.CENTER);
        p2.add(panel1, BorderLayout.NORTH);
        p2.add(searchTemplateViewer, BorderLayout.CENTER);
        searchArea.add(p2, "SEARCH_TEMPLATE_VIEWER");
    } else {
        // Create new split pane splitting the top half of the client pane
        if (ltr) {
            split2 = new JSplitPane( JSplitPane.HORIZONTAL_SPLIT, searchTemplateList,
                               searchTemplateViewer);
        } else {
            split2 = new JSplitPane( JSplitPane.HORIZONTAL_SPLIT, searchTemplateViewer,
                               searchTemplateList);
            // How can you set the resize weight to make this resize apropriately?
        }
        // Add the split pane with the search components to the top
        searchArea.add(split2, "SEARCH_TEMPLATE_VIEWER");
    }
    // Add the searchPanel card
    searchArea.add(searchPanel, "SEARCH_PANEL");
    
        topPane.add(searchArea, BorderLayout.CENTER);

      // Add search results viewer to bottom
      bottomPane.add( BorderLayout.CENTER, searchResultsViewer);

     // register the listeners
     itemAttributesEditor.addEditCompletedListener(this);
     itemAttributesEditor.addEditCompletedListener(versionsViewer);
     itemAttributesEditor.addEditCancelledListener(this);
     logonPanel.addLogonCompletedListener(this);
     logonPanel.addLogonCancelledListener(this);
     searchTemplateList.addTemplateSelectedListener(searchTemplateViewer);
     searchTemplateList.addTemplateSelectedListener(searchResultsViewer);
     searchTemplateViewer.addSearchStartedListener(searchResultsViewer);
     searchTemplateViewer.addSearchResultsListener(searchResultsViewer);
     searchPanel.addSearchStartedListener(searchResultsViewer);
     searchPanel.addSearchResultsListener(searchResultsViewer);
     searchPanel.addEntitySelectedListener(searchResultsViewer);
     searchResultsViewer.addViewDocumentListener(documentViewer);
     searchResultsViewer.addEditItemAttributesListener(this);
     searchResultsViewer.addViewVersionsListener(this);
     versionsViewer.addViewVersionsCancelledListener(this);
     versionsViewer.addViewDocumentListener(documentViewer);
     versionsViewer.addEditItemAttributesListener(this);
     
     setBeanProperties();

   }

   // Not good programming.  This method is here so federation.Handler
   //   (a test tool) can exit when Dialog is closed or Cancel is pressed
   public static void exitProgram() {
      client.exitProgram();
   }
   
   private boolean templatesSupported = false;
   public boolean isTemplatesSupported() {
        return templatesSupported;
   }
   

    /*
     * set bean properties
     **/
   private void setBeanProperties()
   {
      searchTemplateViewer.setLaunchSearchOnEnter ( true);

      searchTemplateViewer.setMaxResults(maxresults);
      searchPanel.setMaxResults(maxresults);
    }

   protected void logoff() { //throws EJMCancelLogoffException {
      // Warning:  Don't call System.exit since logoff is called from
      // an applet (as well as application), and issuing System.exit
      // from an applet will kill ALL browser windows.
      if( documentViewer != null ) {
          documentViewer.terminate();
       }
      try {
       if (connection.isConnected() ) {
          connection.disconnect();
        }
       }catch (Exception exc) {
        System.out.println("Warning: exception occured while logging off of EIP database.");
       }
   }

  /* Show property values
   */
  void showProperties() {
     System.out.println("server:  " + server);

     System.out.println("helpURL:  " + helpURLString);
     
     if (connectionType == CMBBaseConstant.CMB_CONNTYPE_REMOTE) {
        System.out.println("Connection type is REMOTE");
     } else if (connectionType == CMBBaseConstant.CMB_CONNTYPE_LOCAL) {
        System.out.println("Connection type is LOCAL");
     } else {
        System.out.println("Connection type is DYNAMIC");
     }  
     
  }


  protected static void setLookAndFeel(Object caller) {
      try{
         UIManager.setLookAndFeel(UIManager.getSystemLookAndFeelClassName());
       }
       catch (java.lang.IllegalAccessException exc) {
       //EJMTrace.trace(EJMTrace.INFO,caller,exc.toString() + ":" + exc.getMessage());
       }
       catch (java.lang.InstantiationException   exc) {
       //EJMTrace.trace(EJMTrace.INFO,caller,exc.toString() + ":" + exc.getMessage());
       }
       catch (java.lang.ClassNotFoundException  exc) {
       //EJMTrace.trace(EJMTrace.INFO,caller,exc.toString() + ":" + exc.getMessage());
       }
       catch (UnsupportedLookAndFeelException exc) {
       //EJMTrace.trace(EJMTrace.INFO,caller,exc.toString() + ":" + exc.getMessage());
       }
       catch (Exception exc) { }
  }

 /**
 * Creates a dialog using the CMBLogonPanel bean. 
 * Logon/connect to federated server  
 */
 private void logon() {
     if (logonDialog == null) {
     if (client.getMainFrame() != null) {
        client.getMainFrame().setVisible(true);
        logonDialog=new LogonDialog(client.getMainFrame());
     } else
        logonDialog=new LogonDialog();

      logonDialog.setLocation(150,150);
      logonDialog.setVisible(true);
      

   }
 }


public void onEditItemAttributes(CMBEditItemAttributesEvent event) {
   if ( attributesEditorDialog == null ) {
    if (client.getMainFrame() != null)
        attributesEditorDialog=new JDialog(client.getMainFrame());
    else
        attributesEditorDialog=new JDialog();
    attributesEditorDialog.setTitle((String)ResourceMgr.getString("UPDATE_ITEM_ATTRIBUTES"));
    attributesEditorDialog.setModal(true);          
    attributesEditorDialog.getContentPane().setLayout(new BorderLayout());
    }
    itemAttributesEditor.setItem(event.getItem());
    attributesEditorDialog.getContentPane().add(itemAttributesEditor);
    SwingUtilities.updateComponentTreeUI( itemAttributesEditor );
    attributesEditorDialog.pack();
    attributesEditorDialog.setVisible(true);
 }
public void onViewVersions(CMBViewVersionsEvent event) {
   if ( versionsViewerDialog == null ) {
    if (client.getMainFrame() != null) 
        versionsViewerDialog=new JDialog(client.getMainFrame());
    else 
        versionsViewerDialog=new JDialog();
    versionsViewerDialog.setTitle(ResourceMgr.safeGetString("VIEW_ITEM_VERSIONS"));
    versionsViewerDialog.setModal(true);          
    versionsViewerDialog.getContentPane().setLayout(new BorderLayout());
    }
    versionsViewer.setItem(event.getItem());
    versionsViewerDialog.getContentPane().add(versionsViewer);
    SwingUtilities.updateComponentTreeUI( versionsViewer );
    versionsViewerDialog.pack();
    versionsViewerDialog.setVisible(true);
 }
 public void onLogonCompleted(CMBLogonCompletedEvent e) {
    logonDialog.dispose();
    try {
        templatesSupported = false;
        if (connection.getDsType().equals(CMBBaseConstant.CMB_DSTYPE_FED) || connection.getDsType().equals(CMBBaseConstant.CMB_DSTYPE_OD)) {
            // Get templates
            CMBSearchTemplate[] templates = connection.getSchemaManagement().getSearchTemplates();
            // If there are no templates, won't try to use them
            // Entities will be shown instead
            if (templates != null && templates.length > 0) {
                // Display STV if we have templates, SP if not
                // Set the connection
                searchTemplateList.setConnection(connection);
                searchTemplateViewer.setConnection(connection);
                // Show the template card
                ((CardLayout)searchArea.getLayout()).show(searchArea, "SEARCH_TEMPLATE_VIEWER");
                // select the first template 
                searchTemplateList.setSelectedTemplateIndex(0);
                templatesSupported = true;
            }        
        }        
     } 
     // No connection
     catch (CMBNoConnectionException ex) {
           MessageBox mb = new MessageBox(ResourceMgr.getString("TITLEBAR_TITLE"), 50, this);
           mb.setMessageText(ResourceMgr.getString("CONNECTION_FAILURE",connection.getServerName()));
           mb.addChoice(ResourceMgr.getString("OK"),"OK");
           mb.setSynchronousMode(true);
           mb.display();
           return;
        
     }
     // There's a connection, but a problem getting templates
     // Use entities instead
     catch (CMBException ex) {
        templatesSupported = false;
     }
     if (!templatesSupported) {
        // Set the connection
        searchPanel.setConnection(connection);
        // Show the search panel
        ((CardLayout)searchArea.getLayout()).show(searchArea, "SEARCH_PANEL");
        // select the first entity
        try {
        String[] entityNames = connection.getSchemaManagement().getEntityNames();
        if (entityNames.length > 0)
            searchPanel.showEntity(entityNames[0]);
        } catch (CMBException ex2) {
        }
     }
     functionBar.verifyButtonStates();
   
       
 }
 
 

 public void onLogonCancelled(CMBLogonCancelledEvent e)  {
    System.out.println(" Logon has been cancelled, Exiting... " );
    logoff();
    client.exitProgram();
    logonDialog.dispose();
    
   }

   public void onEditCompleted(CMBEditCompletedEvent e)  {
     attributesEditorDialog.dispose();
    }
 
    public void onEditCancelled(CMBEditCancelledEvent e)  {
      attributesEditorDialog.dispose();
     }
    public void onViewVersionsCancelled(CMBViewVersionsCancelledEvent e)  {
      versionsViewerDialog.dispose();
     }
     
    
	public static Image getImage(String name)
	{
		
      Toolkit toolkit= Toolkit.getDefaultToolkit();
      Image image	= null;
      MediaTracker mediaTracker = new MediaTracker(new Panel());
      java.net.URL url = ResourceMgr.getImageURL(name); 
      if(url != null)
			image = toolkit.getImage(url);
      mediaTracker.addImage(image,0);
		try
		{
			mediaTracker.waitForAll();
		}
		catch(Exception ex)
		{
			System.out.println(ex);
		}
		return image;
	}


 // Inner class - logon dialog
 
 private class LogonDialog extends JDialog {
    
    JTextArea label_copyright = new JTextArea() {
        public boolean isFocusTraversable() { return false; }
    };
    ImageIcon icon = new ImageIcon(ClientPane.this.getImage("CC_LOGON")); 
    JLabel label1 = new JLabel("",icon,JLabel.CENTER);
    JPanel jp=new JPanel();
      
    public LogonDialog() {
        this(null);
    }
    
    public LogonDialog(Frame parent) {
        super(parent); // defect 22795
        setTitle(ClientPane.PRODUCT_NAME);
        setModal(true);
        label_copyright.setBackground(label1.getBackground());
        label_copyright.setEditable(false);
        label_copyright.setBorder(new javax.swing.border.EmptyBorder(0,0,0,0));
        label_copyright.setText((String)ResourceMgr.getString("PRODUCT_NAME_NUMBER_VERSION_COPYR"));
        
        getContentPane().setLayout(new PBorderLayout());
        // CMH defect 43771.  Since copyright is not translated, should leave orientation in 
        // ltr (English) format.
        //if (!ltr) {
        //    label_copyright.setComponentOrientation(ComponentOrientation.RIGHT_TO_LEFT);
        //}
        jp.setLayout(new PBorderLayout());
        jp.add(label1,BorderLayout.WEST);
        jp.add(label_copyright,BorderLayout.CENTER);

        // Bold font for application
        Font f=new Font("SansSerif",java.awt.Font.BOLD,12);
        setFont(f);
        logonPanel.setFont(f);
        // Plain font for the copyright for better legibility
        Font f2=new Font("SansSerif",java.awt.Font.PLAIN,12);
        label_copyright.setFont(f2);
        getContentPane().add(jp,BorderLayout.NORTH);
        getContentPane().add(ClientPane.this.logonPanel,BorderLayout.CENTER);

        setDefaultCloseOperation(JFrame.DO_NOTHING_ON_CLOSE );
        addWindowListener(new WindowAdapter() {
            public void windowClosing(WindowEvent e) {
            logoff();
            client.exitProgram();
            dispose();
            }
        });
        pack();
    }
     
 };
 
class PBorderLayout extends java.awt.BorderLayout {
    
    
    public PBorderLayout() {
        super();
    }
    public PBorderLayout(int h, int v) {
        super(h,v);
    }
    
    public void addLayoutComponent(Component c, Object name) {
        // If not left to right, then flip east and west
        if (!ClientPane.isLeftToRight()) {
            if (name.equals(BorderLayout.WEST)) {
                super.addLayoutComponent(c, BorderLayout.EAST);
                return;
            }
            if (name.equals(BorderLayout.EAST)) {
                super.addLayoutComponent(c, BorderLayout.WEST);
                return;
            }
        }
        super.addLayoutComponent(c,name);
    }   
    
}
 

}
