//
/*BEGINPROLOGUE************************************************************************
 * @copyright(disclaimer)                                                             *
 * Licensed Materials - Property of IBM                                               *
 * IBM Enterprise Information Portal for Multiplatforms V8.1 (program number 5724-B43)*
 * (c ) Copyright IBM Corp. 1998,  2002. All Rights Reserved.                         *
 *                                                                                    *
 * US Government Users Restricted Rights                                              *
 * Use, duplication or disclosure restricted by GSA ADP Schedule                      *
 * Contract with IBM Corp.                                                            *
 *                                                                                    *
 * DISCLAIMER OF WARRANTIES :                                                         *
 *                                                                                    *
 * Permission is granted to copy and modify this Sample code, and to                  *
 * distribute modified versions provided that both the copyright                      *
 * notice,- and this permission notice and warranty disclaimer appear in all copies   * 
 * and modified versions.                                                             *
 *                                                                                    *
 * THIS SAMPLE CODE IS LICENSED TO YOU AS-IS.                                         *
 * IBM AND ITS SUPPLIERS AND LICENSORS DISCLAIM ALL WARRANTIES, EITHER EXPRESS OR     *
 * IMPLIED, IN SUCH SAMPLE CODE, INCLUDING THE WARRANTY OF NON-INFRINGEMENT AND THE   *
 * IMPLIED WARRANTIES OF MERCHANTABILITY OR FITNESS FOR A PARTICULAR PURPOSE. IN NO   *
 * EVENT WILL IBM OR ITS LICENSORS OR SUPPLIERS BE LIABLE FOR ANY DAMAGES ARISING OUT *
 * OF THE USE OF OR INABILITY TO USE THE SAMPLE CODE, DISTRIBUTION OF THE SAMPLE CODE,*
 * OR COMBINATION OF THE SAMPLE CODE WITH ANY OTHER CODE. IN NO EVENT SHALL IBM OR ITS*
 * LICENSORS AND SUPPLIERS BE LIABLE FOR ANY LOST REVENUE, LOST PROFITS OR DATA, OR   *
 * FOR DIRECT, INDIRECT, SPECIAL, CONSEQUENTIAL,INCIDENTAL OR PUNITIVE DAMAGES,       *
 * HOWEVER CAUSED AND REGARDLESS OF THE THEORY OF LIABILITY,-, EVEN IF IBM OR ITS     *
 * LICENSORS OR SUPPLIERS HAVE BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGES.       *
 *                                                                                    *
 * @endCopyright                                                                      *
 **************************************************************************************/
//


// sample connect/execute/disconnect to DES

/*
 * Filename       : TExecuteDES.java
 * Description    : Connect to DES Datastore.
 *                  Create query string.
 *                  Specify to retrieve a max result of 5 items.
 *                  Get the result in a resultset cursor.
 *                  Loop thru results.
 *                  Display documents found and their parts and attributes.
 *                  Disconnect from DES Datastore.
 */

import com.ibm.mm.sdk.common.*;
import com.ibm.mm.sdk.server.*;
import java.io.*;
import java.lang.*;
import java.util.*;

// TExecuteDES class
public class TExecuteDES implements DKConstantDES
{


  // Main method
  public static void main(String argv[])
  {

   // Indicate this is a Console Application on Windows NT/95 for error
   // message display in dos console code page. If this was a Windows
   // GUI (Graphic User Interface) Application this would not be necessary
   // because DK_CM_SS_WINDOWS is the default.
   DKEnvironment.setSubSystem(DK_CM_SS_CONSOLE);

      //*** for input parameters
      String libSrv = new String(""),   // library server
             userid = new String(""),     //userid
             pw = new String(""), // pw
             connect_string = "";

      //--------------------------------------------------------------
      // Checking for input parameters
      //--------------------------------------------------------------
      if (argv.length != 4)
      {
       System.out.println("Usage: " );
       System.out.println("java TExecuteDES " +
        "tcp/ipAddress " +
        "userid " +
        "userpw " +
        "PORT=80;DESAPPID=appid;DESAPPPW=apppw[;DESCFGPATH=<des cfg file>;DESLOCALE=<locale>]");
       return;
      }
      else
      {
       libSrv = argv[0];
       userid = argv[1];
       pw = argv[2];
       connect_string = argv[3];
      }

    DKDatastoreDES dsDES = null;
    try {
    dsDES = new DKDatastoreDES();
    dkResultSetCursor pCur = null;
    DKNVPair parms[] = null;
    System.out.println("connecting to datastore");
    dsDES.connect(libSrv,userid,pw,connect_string);
    System.out.println("datastore connected libSrv " + libSrv + " userid " +
                        userid );

    String cmd = "SEARCH=(DATABASE=(Notes 5 Help);COND=((IN \"Subject\" \"key\"));";
    cmd = cmd + "OPTION=(MAX_RESULTS=1;" +
                         "TIME_LIMIT=30))";
    DKDDO ddo = null;
    System.out.println("query string " + cmd);
    System.out.println("executing query");

    pCur = dsDES.execute(cmd,DK_DES_GQL_QL_TYPE,parms);

    System.out.println("cardinality " + pCur.cardinality());
    System.out.println("datastore executed query");
    System.out.println("process query results");
    while (pCur.isValid())
     {
     ddo = pCur.fetchNext();
     if (ddo != null)
       {
       System.out.println("");
       System.out.println("=====> Item " + pCur.getPosition() + " <=====");
       displayDDO(ddo);
       }
     }

    pCur.destroy(); // Finished with the cursor
    System.out.println("query results processed");
    dsDES.disconnect();
    System.out.println("datastore disconnected");
    dsDES.destroy();
    } // end try
    catch (DKException exc)
    {
     try {
     dsDES.destroy();
     }
     catch (Exception e)
     {
      e.printStackTrace();
     }
     System.out.println("Exception name " + exc.name());
     System.out.println("Exception message " + exc.getMessage());
     System.out.println("Exception error code " + exc.errorCode());
     System.out.println("Exception error state " + exc.errorState());
     exc.printStackTrace();
    }
    catch (Exception exc)
    {
     try {
     dsDES.destroy();
     }
     catch (Exception e)
     {
      e.printStackTrace();
     }
     System.out.println("Exception message " + exc.getMessage());
     exc.printStackTrace();
    }
  } // end main


   // method to show DDO hit
   public static void displayDDO(DKDDO ddo)
                throws DKException, Exception
     {
     dkXDO pXDO = null;
     int i = 0;
     int numDataItems = 0;
     short k = 0;
     short j = 0;
     Integer valueCount = null;
     Object value = null;
     String dataName = null;
     dkCollection pCol = null;
     dkIterator pIter = null;
     Object anObject = null;
     String mimeType = "";
     numDataItems = ddo.dataCount();
     DKPid pid = ddo.getPidObject();
     System.out.println("pid string " + pid.pidString());

      System.out.println("Number of Attributes " + numDataItems);
      for (j = 1; j <= numDataItems; j++)
        {
        anObject = ddo.getData(j);
        dataName = ddo.getDataName(j);
        System.out.println(j + ": Name " + dataName );
        // determine if data item has a single value or multiple values
        Short type = (Short)ddo.getDataPropertyByName(j, DK_CM_PROPERTY_TYPE);
        k = type.shortValue();
        if ((k == DK_CM_COLLECTION || k == DK_CM_COLLECTION_XDO)
            && (!dataName.equals(DK_CM_DKPARTS)) )
          {
          pCol = (dkCollection)anObject;
          pIter = pCol.createIterator();
          i = 0;
          while (pIter.more() == true)
            {
            i++;
            value = pIter.next();
            System.out.println("   Value" + i + " " + value);
            }
          } // end DK_CM_COLLECTION
        else
        if (dataName.equals(DK_CM_DKPARTS))  // this is a DKParts with a URL and/or a doclink
          {
          pCol = (dkCollection)anObject;
          pIter = pCol.createIterator();
          while (pIter.more() == true)
            {
            pXDO = (dkXDO) pIter.next();
            mimeType = pXDO.getMimeType();
            if (mimeType.equals("text/html"))
              System.out.println("   URL: " + new String(pXDO.getContent()));
            else if(mimeType.equals("text/doclink"))
              System.out.println("   doclink: " + new String(pXDO.getContent()));
            }
          } // end DK_CM_COLLECTION_XDO
        else
          System.out.println("   Value " + anObject);
        } // end for j
      } // end displayDDO


} // end class

