/*BEGINPROLOGUE************************************************************************
 * @copyright(disclaimer)                                                             *
 * Licensed Materials - Property of IBM                                               *
 * IBM Enterprise Information Portal for Multiplatforms V8.1 (program number 5724-B43)*
 * (c ) Copyright IBM Corp. 1998,  2002. All Rights Reserved.                         *
 *                                                                                    *
 * US Government Users Restricted Rights                                              *
 * Use, duplication or disclosure restricted by GSA ADP Schedule                      *
 * Contract with IBM Corp.                                                            *
 *                                                                                    *
 * DISCLAIMER OF WARRANTIES :                                                         *
 *                                                                                    *
 * Permission is granted to copy and modify this Sample code, and to                  *
 * distribute modified versions provided that both the copyright                      *
 * notice,- and this permission notice and warranty disclaimer appear in all copies   * 
 * and modified versions.                                                             *
 *                                                                                    *
 * THIS SAMPLE CODE IS LICENSED TO YOU AS-IS.                                         *
 * IBM AND ITS SUPPLIERS AND LICENSORS DISCLAIM ALL WARRANTIES, EITHER EXPRESS OR     *
 * IMPLIED, IN SUCH SAMPLE CODE, INCLUDING THE WARRANTY OF NON-INFRINGEMENT AND THE   *
 * IMPLIED WARRANTIES OF MERCHANTABILITY OR FITNESS FOR A PARTICULAR PURPOSE. IN NO   *
 * EVENT WILL IBM OR ITS LICENSORS OR SUPPLIERS BE LIABLE FOR ANY DAMAGES ARISING OUT *
 * OF THE USE OF OR INABILITY TO USE THE SAMPLE CODE, DISTRIBUTION OF THE SAMPLE CODE,*
 * OR COMBINATION OF THE SAMPLE CODE WITH ANY OTHER CODE. IN NO EVENT SHALL IBM OR ITS*
 * LICENSORS AND SUPPLIERS BE LIABLE FOR ANY LOST REVENUE, LOST PROFITS OR DATA, OR   *
 * FOR DIRECT, INDIRECT, SPECIAL, CONSEQUENTIAL,INCIDENTAL OR PUNITIVE DAMAGES,       *
 * HOWEVER CAUSED AND REGARDLESS OF THE THEORY OF LIABILITY,-, EVEN IF IBM OR ITS     *
 * LICENSORS OR SUPPLIERS HAVE BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGES.       *
 *                                                                                    *
 * @endCopyright                                                                      *
 **************************************************************************************/

/*
 * Filename       : TADLsmDemoFed.java
 * Description    : test add, delete, list schema-mappings for DLs
 *                  Need to run server definition mapping first
 *                  TADLServersFed <cmbdb> <userid> <pw> add
 *
 */

import java.sql.*;
import java.lang.*;
import java.util.*;
import com.ibm.mm.sdk.common.dkSchemaMapping;
import com.ibm.mm.sdk.common.dkAttrMapping;
import com.ibm.mm.sdk.common.*;
import com.ibm.mm.sdk.server.*;

class TADLsmDemoFed implements DKConstantFed, DKConstantDL
{
   public static void main(String argv[]) 
   { 
      // checking input parameters
      String libSrv = "",    // library server
             userid = "",    // userid 
             pw     = "",    // pw
            cmd    = "";    // command: add, del, or list
        
      //-------------------------------------------------------------- 
      // Checking for input parameters
      //-------------------------------------------------------------- 
      if (argv.length < 4) {
         System.out.println("Usage Syntax: ");
         System.out.println("\t java TADLsmDemoFed <cmbdb> <userid> <pw> <add/del/list>" );
         return;
      }
      else {
         libSrv = argv[0];
         userid = argv[1];
         pw     = argv[2];
         cmd    = argv[3];
      }   

      try {
         // create federated datastore
         System.out.println(" *** connecting to datastore : " + libSrv);
         dkDatastore ds = new DKDatastoreFed();
         ds.connect(libSrv, userid, pw, "");
         System.out.println(" *** datastore connected *** "); 

         // get the Def
         dkDatastoreDef  defFed = ds.datastoreDef();

         //===================================================================================================
         // Defines federate entity and attribute here.
         //===================================================================================================

         //---------------------------------------------------------------------------------------------------
         // Create 'FedDemo' entity and following attribute:
         //    + FedUserId
         //    + FedTitle
         //    + FedDate
         //    + FedAuthor
         //    + FedSubject
         
         DKEntityDefFed FedDemo = new DKEntityDefFed(ds);        
                     FedDemo.setName("FedDemo");
                     FedDemo.setDescription("FedDemo to map to DLDEMO & DLSAMPLE at LSMUFASA and LSCHIUN");
    
         // create a federated attributes belong to FedDemo:
         dkAttrDef fUserId = new DKAttrDefFed();         // Create FedUserId attribute
                   fUserId.setEntityName("FedDemo");
                   fUserId.setName("FedUserId");
                   fUserId.setType(DK_CM_VSTRING);
                   fUserId.setId((short)95);
                   fUserId.setSize(50);
                   fUserId.setScale(1);
                   fUserId.setPrecision(50);
                   fUserId.setNullable(true);
                   fUserId.setStringType(DK_ATTR_ALPHA);
    
         dkAttrDef fTitle = new DKAttrDefFed();          // Create FedTitle attribute
                   fTitle.setEntityName("FedDemo");
                   fTitle.setName("FedTitle");
                   fTitle.setType(DK_CM_VSTRING);
                   fTitle.setId((short)99);
                   fTitle.setSize(100);
                   fTitle.setScale(1);
                   fTitle.setPrecision(100);
                   fTitle.setNullable(true);
                   fTitle.setStringType(DK_ATTR_ALPHA);
    
         dkAttrDef fDate = new DKAttrDefFed();           // Create FedDate attribute
                   fDate.setEntityName("FedDemo");
                   fDate.setName("FedDate");
                   fDate.setType(DK_CM_VSTRING);
                   fDate.setId((short)100);
                   fDate.setSize(100);
                   fDate.setScale(1);
                   fDate.setPrecision(100);
                   fDate.setNullable(true);
                   fDate.setStringType(DK_ATTR_ALPHA);



         //===================================================================================================
         // Add the federate entity here.
         try {
            if (cmd.equalsIgnoreCase("add")) {
               FedDemo.add();        // adding an entity to the federated database
               FedDemo.add(fUserId);
               FedDemo.add(fTitle);
               FedDemo.add(fDate);
               System.out.println(" *** federated entity and attributes created ");
            }
         }
         catch (DKException exc) {
            System.out.println("Exception name " + exc.name());
            System.out.println("Exception message " + exc.getMessage());
            exc.printStackTrace();
         }
         
         //===================================================================================================
         // Create schema mapping.
         // For each additional back-ends, following steps are required:
         //     a) Create memory native attributes for each index class on each server.
         //     b) Create attributes mapping which map fed attribute with native attributes
         //     c) Create schema mapping object.
         
         
         //---------------------------------------------------------------------------------------------------
         // Create native attributes in memory for DL index class DLDEMO for LSMUFASA.         
         //---------------------------------------------------------------------------------------------------
         dkAttrDef nmUserId = new DKAttrDefDL();           // Create DLUSERID attribute
                   nmUserId.setEntityName("DLDEMO");
                   nmUserId.setName("DLUSERID");
                   nmUserId.setType(DK_CM_VSTRING);
                   nmUserId.setId((short)95);
                   nmUserId.setSize(50);
                   nmUserId.setScale(1);
                   nmUserId.setPrecision(50);
                   nmUserId.setNullable(true);
                   nmUserId.setStringType(DK_ATTR_ALPHA);
     
         dkAttrDef nmTitle = new DKAttrDefDL();            // Create DLTITLE attribute
                   nmTitle.setEntityName("DLDEMO");
                   nmTitle.setName("DLTITLE");
                   nmTitle.setType(DK_CM_VSTRING);
                   nmTitle.setId((short)99);
                   nmTitle.setSize(100);
                   nmTitle.setScale(1);
                   nmTitle.setPrecision(100);
                   nmTitle.setNullable(true);
                   nmTitle.setStringType(DK_ATTR_ALPHA);
     
         dkAttrDef nmDate = new DKAttrDefDL();             // Create DLDATE attribute
                   nmDate.setEntityName("DLDEMO");
                   nmDate.setName("DLDATE");
                   nmDate.setType(DK_CM_VSTRING);
                   nmDate.setId((short)100);
                   nmDate.setSize(100);
                   nmDate.setScale(1);
                   nmDate.setPrecision(100);
                   nmDate.setNullable(true);
                   nmDate.setStringType(DK_ATTR_ALPHA);
         
         //---------------------------------------------------------------------------------------------------
         // Create native attributes in memory for DL index class DLDEMO for LSCHIUN         
         //---------------------------------------------------------------------------------------------------
         dkAttrDef ncUserId = new DKAttrDefDL();           // Create DLUSERID attribute
                   ncUserId.setEntityName("DLDEMO");
                   ncUserId.setName("DLUSERID");
                   ncUserId.setType(DK_CM_VSTRING);
                   ncUserId.setId((short)95);
                   ncUserId.setSize(50);
                   ncUserId.setScale(1);
                   ncUserId.setPrecision(50);
                   ncUserId.setNullable(true);
                   ncUserId.setStringType(DK_ATTR_ALPHA);
     
         dkAttrDef ncTitle = new DKAttrDefDL();            // Create DLTITLE attribute
                   ncTitle.setEntityName("DLDEMO");
                   ncTitle.setName("DLTITLE");
                   ncTitle.setType(DK_CM_VSTRING);
                   ncTitle.setId((short)99);
                   ncTitle.setSize(100);
                   ncTitle.setScale(1);
                   ncTitle.setPrecision(100);
                   ncTitle.setNullable(true);
                   ncTitle.setStringType(DK_ATTR_ALPHA);
     
         dkAttrDef ncDate = new DKAttrDefDL();             // Create DLDATE attribute
                   ncDate.setEntityName("DLDEMO");
                   ncDate.setName("DLDATE");
                   ncDate.setType(DK_CM_VSTRING);
                   ncDate.setId((short)100);
                   ncDate.setSize(100);
                   ncDate.setScale(1);
                   ncDate.setPrecision(100);
                   ncDate.setNullable(true);
                   ncDate.setStringType(DK_ATTR_ALPHA);

         //---------------------------------------------------------------------------------------------------
         // Create attributes mapping which map Fed attribute to native attributes on LSMUFASA.
         //---------------------------------------------------------------------------------------------------
         dkAttrMapping mmapUserId = new DKAttrMappingFed();     // Map 'FedUserId' to 'DLUSERID'
                       mmapUserId.addMappedAttr(fUserId);
                       mmapUserId.addMapToAttr(nmUserId);
     
         dkAttrMapping mmapTitle = new DKAttrMappingFed();     // Map 'FedTitle' to 'DLTITLE'
                       mmapTitle.addMappedAttr(fTitle);
                       mmapTitle.addMapToAttr(nmTitle);
     
         dkAttrMapping mmapDate = new DKAttrMappingFed();       // Map 'FedDate' to 'DLDATE'
                       mmapDate.addMappedAttr(fDate);
                       mmapDate.addMapToAttr(nmDate);

         //---------------------------------------------------------------------------------------------------
         // Create attributes mapping which map Fed attribute to native attributes on LSCHIUN.
         //---------------------------------------------------------------------------------------------------
         dkAttrMapping cmapUserId = new DKAttrMappingFed();     // Map 'FedUserId' to 'DLUSERID'
                       cmapUserId.addMappedAttr(fUserId);
                       cmapUserId.addMapToAttr(ncUserId);
     
         dkAttrMapping cmapTitle = new DKAttrMappingFed();     // Map 'FedTitle' to 'DLTITLE'
                       cmapTitle.addMappedAttr(fTitle);
                       cmapTitle.addMapToAttr(ncTitle);
     
         dkAttrMapping cmapDate = new DKAttrMappingFed();       // Map 'FedDate' to 'DLDATE'
                       cmapDate.addMappedAttr(fDate);
                       cmapDate.addMapToAttr(ncDate);

         //---------------------------------------------------------------------------------------------------
         // Create schema mapping.
         //---------------------------------------------------------------------------------------------------
         // Add LSMUFASA schema mapping
         dkSchemaMapping msm = new DKSchemaMappingFed(ds);
                         msm.setServerName("LSMUFASA");        // Server name. Need to put your server name here.
                         msm.setServerType(DK_DL_DSTYPE);
                         msm.setEntityName("DLDEMO");          // Index name
                         msm.setMappedEntityName("FedDemo");   // Fed entity name
     
         // add attribute mapping to this schema
         msm.addAttrMapping(mmapUserId);
         msm.addAttrMapping(mmapTitle);
         msm.addAttrMapping(mmapDate);

         // Add LSCHIUN schema mapping
         dkSchemaMapping csm = new DKSchemaMappingFed(ds);
                         csm.setServerName("LSCHIUN");           // Server name. Need to put your server name here.
                         csm.setServerType(DK_DL_DSTYPE);
                         csm.setEntityName("DLDEMO");            // Index name
                         csm.setMappedEntityName("FedDemo");     // Fed entity name
     
         // add attribute mapping to this schema
         csm.addAttrMapping(cmapUserId);
         csm.addAttrMapping(cmapTitle);
         csm.addAttrMapping(cmapDate);

         //---------------------------------------------------------------------------------------------------
         // Add or del schema mapping to persistent storage.
         //---------------------------------------------------------------------------------------------------
     
         try {
            if (cmd.equalsIgnoreCase("add")) 
            {
               msm.add();
               System.out.println("*****Schema mapping for LSMUFSA - DLDEMO added");

               csm.add();
               System.out.println("*****Schema mapping for LSCHIUN - DLDEMO added");
            }
            else if (cmd.equalsIgnoreCase("del"))
            {
               msm.del();
               System.out.println("*****Schema mapping for LSMUFSA - DLDEMO deleted");

               csm.del();
               System.out.println("*****Schema mapping for LSCHIUN - DLDEMO deleted");
            }
         }
         catch (DKException exc) {
            System.out.println("Exception name " + exc.name());
            System.out.println("Exception message " + exc.getMessage());
            exc.printStackTrace();
         }

         //---------------------------------------------------------------------------------------------------
         // Delete 'FedDemo' entity.
         //---------------------------------------------------------------------------------------------------
         
         try {
            if (cmd.equalsIgnoreCase("del")) {
               FedDemo.del();
               System.out.println("=====FedDemo entity deleted.");
            }
         }
         catch (DKException exc) {
            System.out.println("Exception name " + exc.name());
            System.out.println("Exception message " + exc.getMessage());
            exc.printStackTrace();
         }
         
         //---------------------------------------------------------------------------------------------------
         // List available entity in the federated database
         //---------------------------------------------------------------------------------------------------
         
         System.out.println("-----------Federated Entities--------------");
         DKSequentialCollection pCol = (DKSequentialCollection) ds.listEntities();
         DKSequentialCollection pCol2;
         dkIterator pIter = pCol.createIterator();
         dkIterator pIter2 = null;
         dkEntityDef FedDef = null;
         dkAttrDef attrDef = null;
         int i = 0, j = 0;
         
         while (pIter.more()) {
            i++;
            FedDef = (dkEntityDef) pIter.next();
            System.out.println("\n++++++Entity Def name [" + i + "] - " + FedDef.getName());
            System.out.println("  Entity Description: " + FedDef.getDescription());
            pCol2 = (DKSequentialCollection) ds.listEntityAttrs(FedDef.getName());
            pIter2 = pCol2.createIterator();
            j = 0;
            while (pIter2.more()) {
               j++;
               attrDef = (dkAttrDef) pIter2.next();
               System.out.println("\n-------Attribute name - " + attrDef.getName());
               System.out.println("     +------datastoreType : " + attrDef.datastoreType());
               System.out.println("     +------attributeOf   : " + attrDef.getEntityName());
               System.out.println("     +------type          : " + attrDef.getType());
               System.out.println("     +------size          : " + attrDef.getSize());
               System.out.println("     +------id            : " + attrDef.getId());
               System.out.println("     +------nullable      : " + attrDef.isNullable());
               System.out.println("     +------precision     : " + attrDef.getPrecision());
               System.out.println("     +------scale         : " + attrDef.getScale());
               System.out.println("     +------stringType    : " + attrDef.getStringType());

            }  
            System.out.println("  " + j + " attr listed for " + FedDef.getName() + " entity name");
         }
         System.out.println(i + " entity def listed");
     
         //---------------------------------------------------------------------------------------------------
         // List schema mappings for this entity.
         //---------------------------------------------------------------------------------------------------

         System.out.println(" List schema mapping names in this entity : " + FedDemo.getName());
         String[] smnames = FedDemo.listSchemaMappingNames();
         
         if (smnames != null) {
            for (i = 0; i < smnames.length; i++)
               System.out.println(" SM name : " + smnames[i]);
         } 
         else 
            System.out.println(" no sm defined in this entity ");

         // retrieve all schema mapping objects for this entity
         dkCollection coll = FedDemo.listSchemaMappings();
         dkIterator iter = null;
         if (coll != null) {
            iter = coll.createIterator();
            System.out.println("\n-- list AttrMapping -- ");
            dkAttrMapping _am;
            dkAttrDef     _ad;
            dkSchemaMapping _sm;
            while (iter.more()) {
               _sm = (dkSchemaMapping) iter.next();
               System.out.println("\n *** Schema Mapping Name : " + _sm.getName());
               dkCollection amCol = _sm.listAttrMappings();
               dkIterator amIter = amCol.createIterator();
               while (amIter.more()) {
                  _am = (dkAttrMapping) amIter.next();
                  System.out.println("\n  ++ AttrMapping Name : " +_am.getName());
                  System.out.println("    -- To Function : " + _am.getToNative());
                  System.out.println("    -- From Function : " + _am.getFromNative());
                  System.out.println(" -- list Mapped Attr : ");
                  int num = _am.numberOfMappedAttr();
                  
                  System.out.println("\t Mapped attr size = " + num);
                  for (i = 0; i < num; i++) {
                     _ad = (dkAttrDef)_am.getMappedAttr((short)(i+1));
 
                     System.out.println("\n\t-------Attribute name - " + _ad.getName());
                     System.out.println("\t     +------attributeOf   : " + _ad.getEntityName());
                     System.out.println("\t     +------type          : " + _ad.getType());
                     System.out.println("\t     +------size          : " + _ad.getSize());
                     System.out.println("\t     +------id            : " + _ad.getId());
                     System.out.println("\t     +------nullable      : " + _ad.isNullable());
                     System.out.println("\t     +------precision     : " + _ad.getPrecision());
                     System.out.println("\t     +------scale         : " + _ad.getScale());
                     System.out.println("\t     +------stringType    : " + _ad.getStringType());
                     
                  }
                      
                  System.out.println(" -- list MapTo Attr -- ");
                  num = _am.numberOfMapToAttr();
                  System.out.println("\t MapTo attr size = " + num);
                  for (i = 0; i < num; i++) {
                     _ad = (dkAttrDef)_am.getMapToAttr((short)(i+1));

                     System.out.println("\n\t-------Attribute name - " + _ad.getName());
                     System.out.println("\t     +------attributeOf   : " + _ad.getEntityName());
                     System.out.println("\t     +------type          : " + _ad.getType());
                     System.out.println("\t     +------size          : " + _ad.getSize());
                     System.out.println("\t     +------id            : " + _ad.getId());
                     System.out.println("\t     +------nullable      : " + _ad.isNullable());
                     System.out.println("\t     +------precision     : " + _ad.getPrecision());
                     System.out.println("\t     +------scale         : " + _ad.getScale());
                     System.out.println("\t     +------stringType    : " + _ad.getStringType());
                  }  
               }
            }  
         } 
         else 
            System.out.println(" schema mapping is null ");

         ds.disconnect();
         System.out.println(" ** disconnects... ");
         System.out.println("Done");
      } 
      catch (java.lang.Exception ex) 
      { 
         ex.printStackTrace(); 
      } 
   } 
}





