/*BEGINPROLOGUE************************************************************************
 * @copyright(disclaimer)                                                             *
 * Licensed Materials - Property of IBM                                               *
 * IBM Enterprise Information Portal for Multiplatforms V8.1 (program number 5724-B43)*
 * (c ) Copyright IBM Corp. 1998,  2002. All Rights Reserved.                         *
 *                                                                                    *
 * US Government Users Restricted Rights                                              *
 * Use, duplication or disclosure restricted by GSA ADP Schedule                      *
 * Contract with IBM Corp.                                                            *
 *                                                                                    *
 * DISCLAIMER OF WARRANTIES :                                                         *
 *                                                                                    *
 * Permission is granted to copy and modify this Sample code, and to                  *
 * distribute modified versions provided that both the copyright                      *
 * notice,- and this permission notice and warranty disclaimer appear in all copies   * 
 * and modified versions.                                                             *
 *                                                                                    *
 * THIS SAMPLE CODE IS LICENSED TO YOU AS-IS.                                         *
 * IBM AND ITS SUPPLIERS AND LICENSORS DISCLAIM ALL WARRANTIES, EITHER EXPRESS OR     *
 * IMPLIED, IN SUCH SAMPLE CODE, INCLUDING THE WARRANTY OF NON-INFRINGEMENT AND THE   *
 * IMPLIED WARRANTIES OF MERCHANTABILITY OR FITNESS FOR A PARTICULAR PURPOSE. IN NO   *
 * EVENT WILL IBM OR ITS LICENSORS OR SUPPLIERS BE LIABLE FOR ANY DAMAGES ARISING OUT *
 * OF THE USE OF OR INABILITY TO USE THE SAMPLE CODE, DISTRIBUTION OF THE SAMPLE CODE,*
 * OR COMBINATION OF THE SAMPLE CODE WITH ANY OTHER CODE. IN NO EVENT SHALL IBM OR ITS*
 * LICENSORS AND SUPPLIERS BE LIABLE FOR ANY LOST REVENUE, LOST PROFITS OR DATA, OR   *
 * FOR DIRECT, INDIRECT, SPECIAL, CONSEQUENTIAL,INCIDENTAL OR PUNITIVE DAMAGES,       *
 * HOWEVER CAUSED AND REGARDLESS OF THE THEORY OF LIABILITY,-, EVEN IF IBM OR ITS     *
 * LICENSORS OR SUPPLIERS HAVE BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGES.       *
 *                                                                                    *
 * @endCopyright                                                                      *
 **************************************************************************************/


/*
 * Filename       : TExecMSTDemoFed.java
 * Description    : Create search template in memory and execute a query.
 *                : Need to run the following program first
 *                 java TADLServersFed <cmbdb> <userid> <pw> add
 *                 java TADLsmFed      <cmbdb> <userid> <pw> add
 *
 *
 */

import com.ibm.mm.sdk.server.*;
import com.ibm.mm.sdk.common.*;
import java.io.*;

public class TExecMSTDemoFed implements DKConstantFed 
{

   // Main method
   public static void main(String argv[]){

      // Indicate this is a Console Application on Windows NT/95 for error
      // message display in dos console code page. If this was a Windows
      // GUI (Graphic User Interface) Application this would not be necessary
      // because DK_CM_SS_WINDOWS is the default.
      DKEnvironment.setSubSystem(DK_CM_SS_CONSOLE);
        
      // checking input parameters
      String libSrv = "",    // library server
             userid = "",    // userid 
             pw     = "";    // pw
        
      //-------------------------------------------------------------- 
      // Checking for input parameters
      //-------------------------------------------------------------- 
      if (argv.length < 3) {
         System.out.println("Usage Syntax: ");
         System.out.println("\t java TExecMSTDemoFed <cmbdb> <userid> <pw>" );
         return;
      }
      else {
         libSrv = argv[0];
         userid = argv[1];
         pw     = argv[2];
      }   

        
        
      try {
         // create federated datastore
         System.out.println(" *** connecting to datastore : " + libSrv);
         DKDatastoreFed ds = new DKDatastoreFed();
         ds.connect(libSrv, userid, pw, "");
         System.out.println(" *** datastore connected *** "); 

         // get the Def
         dkDatastoreDef  defFed = ds.datastoreDef();
          
         // set up search criteria memory variable.
         short[] ops = { DKOpCode.EQ, DKOpCode.NEQ, DKOpCode.LIKE, DKOpCode.NOTLIKE, DKOpCode.IN, DKOpCode.NOTIN };

         DKNVPair opt[] = new DKNVPair[1];
         opt[0] = new DKNVPair("MAX_RESULTS", "6");

         //===================================================================================================
         // Defines search criterias for FedDemo
         //===================================================================================================
            
         // Create search criteria for FedTitle
         DKSearchCriteria scTitle = new DKSearchCriteria("FedDemo", "FedTitle");
                          scTitle.setName("Title");
                          scTitle.setDisplayName("Document Title");
                          scTitle.setDefaultOperator(DKOpCode.LIKE);
                          scTitle.setValidOperators(ops);
                          scTitle.setDisplayWidth((short)40);
                          scTitle.setQueryable(true);
                          scTitle.setDisplayable(true);
                          scTitle.setOperator(DKOpCode.EQ);
                          scTitle.setValue("'BASUKI'");
 
         // Create search criteria for FedUserID
         DKSearchCriteria scUserID = new DKSearchCriteria("FedDemo", "FedUserId");
                          scUserID.setName("UserID");
                          scUserID.setDisplayName("Document UserID");
                          scUserID.setDefaultOperator(DKOpCode.LIKE);
                          scUserID.setValidOperators(ops);
                          scUserID.setDisplayWidth((short)40);
                          scUserID.setQueryable(true);
                          scUserID.setDisplayable(true);
                          scUserID.setOperator(DKOpCode.LIKE);
                          scUserID.setValue("'Progr%'");

         // Create search criteria for FedDate
         DKSearchCriteria scDate = new DKSearchCriteria("FedDemo", "FedDate");
                          scDate.setName("Date");
                          scDate.setDisplayName("Document Date");
                          scDate.setDefaultOperator(DKOpCode.LIKE);
                          scDate.setValidOperators(ops);
                          scDate.setDisplayWidth((short)40);
                          scDate.setQueryable(true);
                          scDate.setDisplayable(true);
                          scDate.setOperator(DKOpCode.NEQ);
                          scDate.setValue("NULL");

         //===================================================================================================
         // Defines memory only search templates for FedDemo
         //===================================================================================================
         
         // Create search template: FedUserId AND FedTitle AND FedDate
         DKSearchTemplateFed stANDS = new DKSearchTemplateFed(ds);
                             stANDS.setName("FedDemoST_ANDS");
                             stANDS.setDescription("All ANDS in memory search template for FedDemo.");
                             stANDS.setEntityName("FedDemo");
                             stANDS.setParametricOp(DK_CM_OP_ALL_ANDS);
                             stANDS.setOptionList(opt);
                             
         stANDS.addCriteria(scUserID);     // Add search criterias to memory only.
         stANDS.addCriteria(scTitle);
         stANDS.addCriteria(scDate);
                                       
              
         // Create search template: FedUserId OR FedTitle OR FedDate
         DKSearchTemplateFed stORS = new DKSearchTemplateFed(ds);
                             stORS.setName("FedDemoST_ORS");
                             stORS.setDescription("All ORS in memory search template for FedDemo.");
                             stORS.setEntityName("FedDemo");
                             stORS.setParametricOp(DK_CM_OP_ALL_ORS);
                             stORS.setOptionList(opt);
                             
         stORS.addCriteria(scUserID);     // Add search criterias to memory only.
         stORS.addCriteria(scTitle);
         stORS.addCriteria(scDate);


         //===================================================================================================
         // Execute search template.
         //===================================================================================================

         // Execute ALL ANDS search template
         DKCQExpr cqe = stANDS.createQueryExpr();

         System.out.println(" *** execute query expression for all ANDS search template ****");
         System.out.println("    Criteria: FedUserID AND FedTitle AND FedDate");
         dkResultSetCursor rsc = ds.execute(cqe);

         //  get result set cursor
         if (rsc == null) 
            System.out.println(" *** cursor is null");
         else {
	         System.out.println(" Search result hit count = " + rsc.cardinality());
            DKDDO ddo;

            while (rsc.isValid()  && !(rsc.isEnd()) )  {
               System.out.println("before fetch, position = " + rsc.getPosition() );
               ddo = rsc.fetchNext();
               System.out.println("after fetch, position = " + rsc.getPosition() );
    
               if (ddo != null) {  
                  System.out.println(" pidString : " + ddo.getPidObject().pidString());
                  displayDDO(ddo);
               } 
            }
            rsc.destroy();
         }         
         
         // Execute ALL ORS search template
         cqe = stORS.createQueryExpr();

         System.out.println(" *** execute query expression for all ORS search template ****");
         System.out.println("    Criteria: FedUserID OR FedTitle OR FedDate");
         
         rsc = ds.execute(cqe);

         //  get result set cursor
         if (rsc == null) 
            System.out.println(" *** cursor is null");
         else {
	         System.out.println(" Search result hit count = " + rsc.cardinality());
            DKDDO ddo;

            while (rsc.isValid()  && !(rsc.isEnd()) )  {
               System.out.println("before fetch, position = " + rsc.getPosition() );
               ddo = rsc.fetchNext();
               System.out.println("after fetch, position = " + rsc.getPosition() );
    
               if (ddo != null) {  
                  System.out.println(" pidString : " + ddo.getPidObject().pidString());
                  displayDDO(ddo);
               } 
            }
            rsc.destroy();
         }         

         
         ds.disconnect();
         System.out.println("Disconnect. Done.");
         ds.destroy();
      }    
      catch (DKException exc) {
         System.out.println("Exception name " + exc.name());
         System.out.println("Exception message " + exc.getMessage());
         exc.printStackTrace();
      }
      catch (Exception exc) {
         System.out.println("Exception message " + exc.getMessage());
         exc.printStackTrace();
      }

   }

   public static void displayDDO(DKDDO ddo) throws DKException, Exception
   {
      String   dataName;
      Object a = null;
 
      if (ddo != null) {
         for (short i = 1; i <= ddo.dataCount(); i++) {
            a = ddo.getData(i);
            dataName = ddo.getDataName(i);
            System.out.print(i + ": Attribute Name :" + dataName );
            if (a != null)
               System.out.println(",\t   Value :" + a.toString());    
            else
               System.out.println(",\t   Value : null ");    
         }  
      }
   }

}

