/*BEGINPROLOGUE************************************************************************
 * @copyright(disclaimer)                                                             *
 * Licensed Materials - Property of IBM                                               *
 * IBM Enterprise Information Portal for Multiplatforms V8.1 (program number 5724-B43)*
 * (c ) Copyright IBM Corp. 1998,  2002. All Rights Reserved.                         *
 *                                                                                    *
 * US Government Users Restricted Rights                                              *
 * Use, duplication or disclosure restricted by GSA ADP Schedule                      *
 * Contract with IBM Corp.                                                            *
 *                                                                                    *
 * DISCLAIMER OF WARRANTIES :                                                         *
 *                                                                                    *
 * Permission is granted to copy and modify this Sample code, and to                  *
 * distribute modified versions provided that both the copyright                      *
 * notice,- and this permission notice and warranty disclaimer appear in all copies   * 
 * and modified versions.                                                             *
 *                                                                                    *
 * THIS SAMPLE CODE IS LICENSED TO YOU AS-IS.                                         *
 * IBM AND ITS SUPPLIERS AND LICENSORS DISCLAIM ALL WARRANTIES, EITHER EXPRESS OR     *
 * IMPLIED, IN SUCH SAMPLE CODE, INCLUDING THE WARRANTY OF NON-INFRINGEMENT AND THE   *
 * IMPLIED WARRANTIES OF MERCHANTABILITY OR FITNESS FOR A PARTICULAR PURPOSE. IN NO   *
 * EVENT WILL IBM OR ITS LICENSORS OR SUPPLIERS BE LIABLE FOR ANY DAMAGES ARISING OUT *
 * OF THE USE OF OR INABILITY TO USE THE SAMPLE CODE, DISTRIBUTION OF THE SAMPLE CODE,*
 * OR COMBINATION OF THE SAMPLE CODE WITH ANY OTHER CODE. IN NO EVENT SHALL IBM OR ITS*
 * LICENSORS AND SUPPLIERS BE LIABLE FOR ANY LOST REVENUE, LOST PROFITS OR DATA, OR   *
 * FOR DIRECT, INDIRECT, SPECIAL, CONSEQUENTIAL,INCIDENTAL OR PUNITIVE DAMAGES,       *
 * HOWEVER CAUSED AND REGARDLESS OF THE THEORY OF LIABILITY,-, EVEN IF IBM OR ITS     *
 * LICENSORS OR SUPPLIERS HAVE BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGES.       *
 *                                                                                    *
 * @endCopyright                                                                      *
 **************************************************************************************/

/*
 * Filename       : TExecQryStrExprPQorTQFed.java
 * Description    : test execute federated query (parametric OR text combined) using query string
 * Usuage         : Run these java files first
 *                  TADLServersFed       <cmbdb> <userid> <pw> add
 *                  TADLsmSampleFed      <cmbdb> <userid> <pw> add
 *                  TADLsmTM             <cmbdb> <userid> <pw> add
 *
 */

import com.ibm.mm.sdk.common.*;
import com.ibm.mm.sdk.server.*;
import java.io.*;
import java.sql.*;
import java.util.*;

public class TExecQryStrExprPQorTQFed implements DKConstantFed {

   // Main method
   public static void main(String argv[]){

      // Indicate this is a Console Application on Windows NT/95 for error
      // message display in dos console code page. If this was a Windows
      // GUI (Graphic User Interface) Application this would not be necessary
      // because DK_CM_SS_WINDOWS is the default.
      DKEnvironment.setSubSystem(DK_CM_SS_CONSOLE);

      //--------------------------------------------------------------
      // Checking for input parameters
      //--------------------------------------------------------------
      if (argv.length < 3) {
         System.out.println("Usage Syntax: ");
         System.out.println("\t java TExecQryStrExprPQorTQFed <cmbdb> <userid> <pw> " );
         return;
      }

      String libSrv = argv[0];
      String userid = argv[1];
      String pw     = argv[2];

      try {
         DKDatastoreFed dsFed = new DKDatastoreFed();
         System.out.println("connecting to datastore");
         dsFed.connect(libSrv, userid, pw, "");
         System.out.println("datastore connected");
         String userName = dsFed.userName();
         String dsName = dsFed.datastoreName();
         System.out.println("user " + userName + " dsName " + dsName);

         // prepare parametric query string
         String qryString1 =  "PARAMETRIC_SEARCH = (ENTITY = FedSample, MAX_RESULTS = 6,";
                qryString1 += "COND = (((FedAuthor == 'Alan') OR (FedTitle LIKE 'Program%')) AND (FedSubject <> NULL))";
                qryString1 += "); OPTION = (CONTENT = ATTRONLY)";

         // prepare text query string
         String qryString2 =  "TEXT_SEARCH=(COND=('UNIX' OR 'IBM' OR 'system' OR 'content' OR 'digital'));";
                qryString2 += "OPTION=(SEARCH_INDEX=(FedTMINDEX);MAX_RESULTS=2;)";

         // create combined query string: PQ OR TQ
         String qryString = qryString1 + " OR " + qryString2;

         System.out.println("Query string = " + qryString);
         System.out.println("Executing query string...");
         dkResultSetCursor rsc = dsFed.execute(qryString, DK_CM_FEDERATED_QL_TYPE, null);

         //  get result set cursor
         if (rsc == null)
            System.out.println(" *** cursor is null");
         else {
	         System.out.println(" Search result hit count = " + rsc.cardinality());
            DKDDO ddo;

            while (rsc.isValid()  && !(rsc.isEnd()) )  {
               System.out.println("before fetch, position = " + rsc.getPosition() );
               ddo = rsc.fetchNext();
               System.out.println("after fetch, position = " + rsc.getPosition() );

               if (ddo != null) {
                  System.out.println(" pidString : " + ddo.getPidObject().pidString());
                  displayDDO(ddo);
               }
            }
            rsc.destroy();
         }
         dsFed.disconnect();
         dsFed.destroy();
      }
      catch (DKException exc) {
         System.out.println("Exception name " + exc.name());
         System.out.println("Exception message " + exc.getMessage());
         exc.printStackTrace();
      }
      catch (Exception exc) {
         System.out.println("Exception message " + exc.getMessage());
         exc.printStackTrace();
      }

   }

   public static void displayDDO(DKDDO ddo) throws DKException, Exception
   {
      String   dataName;
      Object a = null;

      if (ddo != null) {
         for (short i = 1; i <= ddo.dataCount(); i++) {
            a = ddo.getData(i);
            dataName = ddo.getDataName(i);
            System.out.print(i + ": Attribute Name :" + dataName );
            if (a == null)
               System.out.println(",\t   Value : is null ");
            else
               System.out.println(",\t   Value :" + a.toString());
         }
      }
   }
}













