/*BEGINPROLOGUE************************************************************************
 * @copyright(disclaimer)                                                             *
 * Licensed Materials - Property of IBM                                               *
 * IBM Information Integrator for Content V8.2  (program number 5724-B43)             *
 * (c ) Copyright IBM Corp. 1998, 2002, 2003. All Rights Reserved.                    *
 *                                                                                    *
 * US Government Users Restricted Rights                                              *
 * Use, duplication or disclosure restricted by GSA ADP Schedule                      *
 * Contract with IBM Corp.                                                            *
 *                                                                                    *
 * DISCLAIMER OF WARRANTIES :                                                         *
 *                                                                                    *
 * Permission is granted to copy and modify this Sample code, and to                  *
 * distribute modified versions provided that both the copyright                      *
 * notice,- and this permission notice and warranty disclaimer appear in all copies   * 
 * and modified versions.                                                             *
 *                                                                                    *
 * THIS SAMPLE CODE IS LICENSED TO YOU AS-IS.                                         *
 * IBM AND ITS SUPPLIERS AND LICENSORS DISCLAIM ALL WARRANTIES, EITHER EXPRESS OR     *
 * IMPLIED, IN SUCH SAMPLE CODE, INCLUDING THE WARRANTY OF NON-INFRINGEMENT AND THE   *
 * IMPLIED WARRANTIES OF MERCHANTABILITY OR FITNESS FOR A PARTICULAR PURPOSE. IN NO   *
 * EVENT WILL IBM OR ITS LICENSORS OR SUPPLIERS BE LIABLE FOR ANY DAMAGES ARISING OUT *
 * OF THE USE OF OR INABILITY TO USE THE SAMPLE CODE, DISTRIBUTION OF THE SAMPLE CODE,*
 * OR COMBINATION OF THE SAMPLE CODE WITH ANY OTHER CODE. IN NO EVENT SHALL IBM OR ITS*
 * LICENSORS AND SUPPLIERS BE LIABLE FOR ANY LOST REVENUE, LOST PROFITS OR DATA, OR   *
 * FOR DIRECT, INDIRECT, SPECIAL, CONSEQUENTIAL,INCIDENTAL OR PUNITIVE DAMAGES,       *
 * HOWEVER CAUSED AND REGARDLESS OF THE THEORY OF LIABILITY,-, EVEN IF IBM OR ITS     *
 * LICENSORS OR SUPPLIERS HAVE BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGES.       *
 *                                                                                    *
 * @endCopyright                                                                      *
 **************************************************************************************

When Deleting Reference Attributes, use it as an Attribute Group.  Deleting Reference 
Attribute Definitions work exactly the same as Attribute Groups.

NOTE:  When retrieving Attribute Group Definitions, if it does not exist, null is returned.  
       In this sample, if null is detected, no action will take place for that particular 
       Attribute Group Definition because it was not defined.

********************************************************************/

// Imports
import com.ibm.mm.sdk.common.*;
import com.ibm.mm.sdk.server.*;
import java.io.*;

/************************************************************************************************
 *          FILENAME: SReferenceAttrDefDeletionICM.java
 *                    ---------------------------------------------------------------------------
 *       DESCRIPTION: Delete Reference Attribute Definitions, Insurance Model Reference Attribute
 *                    Definitions, and Other Sample Reference Attribute Definitions.
 *                    ---------------------------------------------------------------------------
 *     DEMONSTRATION: Deleting Reference Attribute Definitions
 *                    Deleting Insurance Model Reference Attribute
 *                    Deletes all Reference Attributes Defined in SReferenceAttrDefCreationICM
 *                    ---------------------------------------------------------------------------
 * COMMANDLINE USAGE: java SReferenceAttrDefDeletionICM <database> <userName> <password>
 *                    ---------------------------------------------------------------------------
 *      PREREQUISITS: Reference Attributes used here should be already defined.  If not, Sample
 *                    SReferenceAttrDefCreationICM may be run.  Sample will complete whether
 *                    the Reference Attributes exist or not.
 *                    ---------------------------------------------------------------------------
 *    FILES REQUIRED: SConnectDisconnectICM.java
 ************************************************************************************************/
public class SReferenceAttrDefDeletionICM{
    
    //-------------------------------------------------------------
    // Main
    //-------------------------------------------------------------
    /**
     * Run the Sample.
     * @param argv[] String Array containing arguments.  Optional arguments are <databse> <userName> <password>.
     */
    public static void main(String argv[]) throws DKException, Exception{
    
        // Defaults for connecting to the database.
        String database = SConnectDisconnectICM.DEFAULT_DATABASE;
        String userName = SConnectDisconnectICM.DEFAULT_USERNAME;
        String password = SConnectDisconnectICM.DEFAULT_PASSWORD;
        
        //------------------------------------------------------------
        // Checking for input parameters
        //--------------------------------------------------------------
        if (argv.length < 3) { // if not all 3 arguments were specified, use defaults and report correct usage.
            System.out.println("Usage: " );
            System.out.println("  java SReferenceAttrDefDeletionICM <database> <userName> <password>" );
            System.out.println("  *** Some parameters not specified, using defaults..." );
            System.out.println("");
        } else {  // otherwise enough parameters were specified, use the first 3 for the 3 parameters.
            if (argv.length > 0) database = argv[0];
            if (argv.length > 1) userName = argv[1];
            if (argv.length > 2) password = argv[2];
        }//end else
        String ver = SConnectDisconnectICM.EIP_VERSION;
        
        System.out.println("=============================================");
        System.out.println("IBM Information Integrator for Content   v"+ver);
        System.out.println("Sample Program:  SReferenceAttrDefDeletionICM");
        System.out.println("---------------------------------------------");
        System.out.println(" Database: "+database);
        System.out.println(" UserName: "+userName);
        System.out.println("=============================================\n");
        
        try{
            //-------------------------------------------------------------
            // Connect to datastore
            //-------------------------------------------------------------
            // See Sample SConnectDisconnectICM for more information
            System.out.println("Connecting to datastore (Database '"+database+"', UserName '"+userName+"')...");

                DKDatastoreICM dsICM = new DKDatastoreICM();  // Create new datastore object.
                dsICM.connect(database,userName,password,""); // Connect to the datastore.

            System.out.println("Connected to datastore (Database '"+dsICM.datastoreName()+"', UserName '"+dsICM.userName()+"').");

            //--------------------------------------------------------------
            // Retrieve a Reference Attribute (Group)
            //--------------------------------------------------------------
            // Treat the Reference Attribute (Group) as an actual
            // Attribute Group, since it was created as such.
            //
            // Retrieving the "S_autoClaim" Reference Attribute (Group) Definition.
            System.out.println("Retrieving Reference Atttribute 'S_autoClaim'...");

                DKDatastoreDefICM dsDefICM = (DKDatastoreDefICM) dsICM.datastoreDef(); // Get the datastore definition object from the connected datastore.
                DKAttrGroupDefICM attrGroupDef = (DKAttrGroupDefICM) dsDefICM.retrieveAttrGroup("S_autoClaim"); // Retrieve the definition.

            System.out.println("Retrieved Reference Attribute 'S_autoClaim'.");

            //--------------------------------------------------------------
            // Delete Reference Attribute Definition in Datastore Definition
            //--------------------------------------------------------------
            // The Attribute Group Definition's del() method can be called
            // as well as the Datastore Definition's del(attrGroupDef) method.
            //
            // This example will use the Attribute Group's del() method.
            System.out.println("Deleting Reference Attribute Definition 'S_autoClaim' in Datastore...");

                if(attrGroupDef!=null) // if it is not already deleted...
                    attrGroupDef.del();  // Delete Attribute Group Definition in Datastore Definition

            System.out.println("Deleted Reference Attribute Definition 'S_autoClaim' in Datastore.");

            //-------------------------------------------------------------
            // Retrieve & Delete "S_myRef"
            //-------------------------------------------------------------
            // This method uses the Datastore Definition's del(attrGroup) method.
            System.out.println("Retrieving & Deleting Reference Attribute Definition 'S_myRef'...");

                attrGroupDef = (DKAttrGroupDefICM) dsDefICM.retrieveAttrGroup("S_myRef"); // Retrieve the definition.

                if(attrGroupDef!=null) // if it is not already deleted...
                    dsDefICM.del(attrGroupDef);  // Delete Attribute Group Definition in Datastore Definition

            System.out.println("Deleted Reference Attribute Definition 'S_myRef'.");

            //-------------------------------------------------------------
            // Disconnect from datastore & Destroy Reference
            //-------------------------------------------------------------
            // See Sample SConnectDisconnectICM for more information
            System.out.println("Disconnecting from datastore & destroying reference...");            

                dsICM.disconnect();
                dsICM.destroy();

            System.out.println("Disconnected from datastore & destroying reference.");

            //-------------------------------------------------------------
            // Sample program completed without exception
            //-------------------------------------------------------------
            System.out.println("\n==========================================");
            System.out.println("Sample program completed.");
            System.out.println("==========================================\n");
        }
        //------------------------------------------------------------
        // Catch & Print Exceptions        
        //------------------------------------------------------------
        catch (DKException exc){
            SConnectDisconnectICM.printException(exc);  // Print the exception using the function listed below.
        } catch (Exception exc) {
            SConnectDisconnectICM.printException(exc);  // Print the exception using the function listed below.
        }
    }// end main

    //=================================================================
    // Wrapper Functions
    //=================================================================
    // The following are wrapper functions for functionality covered
    // in this sample.  These functions may be used by other samples.

   /**
    * Delete the specified Reference Attribute Definition (Same function as in SAttributeGroupDefDeletionICM)
    * @param dsICM              Connected DKDatastoreICM object.
    * @param referenceAttrName  Reference Attribute Name for the Reference Attribute Definition to Delete.
    **/
    public static void deleteReferenceAttrDef(DKDatastoreICM dsICM, String referenceAttrName) throws DKException, Exception{
        System.out.println("Deleting the Reference Attribute Definition '"+referenceAttrName+"'...");

            // First, we need acces to the Datastore Definition object.
            DKDatastoreDefICM dsDefICM = (DKDatastoreDefICM) dsICM.datastoreDef(); // Get the datastore definition object from the connected datastore.
            DKAttrGroupDefICM attrGroupDef = (DKAttrGroupDefICM) dsDefICM.retrieveAttrGroup(referenceAttrName); // Retrieve the definition.
            if(attrGroupDef!=null) // if it is not already deleted...
                attrGroupDef.del();   // Delete the Attribute Group Definition

        System.out.println("Deleted the Reference Attribute Definition '"+referenceAttrName+"'.");
    }
   
}//end class SReferenceAttrDefDeletionICM
              
