/*BEGINPROLOGUE************************************************************************
 * @copyright(disclaimer)                                                             *
 * Licensed Materials - Property of IBM                                               *
 * IBM Information Integrator for Content V8.2  (program number 5724-B43)             *
 * (c ) Copyright IBM Corp. 1998, 2002, 2003. All Rights Reserved.                    *
 *                                                                                    *
 * US Government Users Restricted Rights                                              *
 * Use, duplication or disclosure restricted by GSA ADP Schedule                      *
 * Contract with IBM Corp.                                                            *
 *                                                                                    *
 * DISCLAIMER OF WARRANTIES :                                                         *
 *                                                                                    *
 * Permission is granted to copy and modify this Sample code, and to                  *
 * distribute modified versions provided that both the copyright                      *
 * notice,- and this permission notice and warranty disclaimer appear in all copies   * 
 * and modified versions.                                                             *
 *                                                                                    *
 * THIS SAMPLE CODE IS LICENSED TO YOU AS-IS.                                         *
 * IBM AND ITS SUPPLIERS AND LICENSORS DISCLAIM ALL WARRANTIES, EITHER EXPRESS OR     *
 * IMPLIED, IN SUCH SAMPLE CODE, INCLUDING THE WARRANTY OF NON-INFRINGEMENT AND THE   *
 * IMPLIED WARRANTIES OF MERCHANTABILITY OR FITNESS FOR A PARTICULAR PURPOSE. IN NO   *
 * EVENT WILL IBM OR ITS LICENSORS OR SUPPLIERS BE LIABLE FOR ANY DAMAGES ARISING OUT *
 * OF THE USE OF OR INABILITY TO USE THE SAMPLE CODE, DISTRIBUTION OF THE SAMPLE CODE,*
 * OR COMBINATION OF THE SAMPLE CODE WITH ANY OTHER CODE. IN NO EVENT SHALL IBM OR ITS*
 * LICENSORS AND SUPPLIERS BE LIABLE FOR ANY LOST REVENUE, LOST PROFITS OR DATA, OR   *
 * FOR DIRECT, INDIRECT, SPECIAL, CONSEQUENTIAL,INCIDENTAL OR PUNITIVE DAMAGES,       *
 * HOWEVER CAUSED AND REGARDLESS OF THE THEORY OF LIABILITY,-, EVEN IF IBM OR ITS     *
 * LICENSORS OR SUPPLIERS HAVE BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGES.       *
 *                                                                                    *
 * @endCopyright                                                                      *
 **************************************************************************************

Please refer to the documentation in SResourceMgrDefCreationICM for concept documentation.

********************************************************************/

// Imports
import com.ibm.mm.sdk.common.*;
import com.ibm.mm.sdk.server.*;
import java.io.*;

/************************************************************************************************
 *          FILENAME: SResourceMgrDefRetrievalICM.java
 *                    ---------------------------------------------------------------------------
 *       DESCRIPTION: Resource Manager Definition, Retrieval, Listing, & Printing.
 *                    Checking for RM Definitions.
 *                    ---------------------------------------------------------------------------
 *     DEMONSTRATION: Accessing Datastore Administration
 *                    Accessing Configuration Management
 *                    Accessing RM Configuration Management
 *                    List Resource Managers in System
 *                    Retrieving Resource Manager Definitions
 *                    Checking to See if RM Definition is Setup
 *                    Print Resource Manager Definition
 *                    ---------------------------------------------------------------------------
 * COMMANDLINE USAGE  java SResourceMgrDefRetrievalICM <Existing RM User-Defined Reference Name>
 *                                                     <database> <userName> <password>
 *                   
 *                    NOTE: The specified reference name must be already defined, such as the
 *                          default "RMDB" specified during install.  If a new RM definition 
 *                          is to be configured, please refer to SResourceMgrDefCreationICM 
 *                          Sample.  This name should consist of all upper-case letters.
 * 
 *                    NOTE: Defaults will be used if the database, username, & password are not 
 *                          specified.  All others are required.
 *
 *                    ex: java SResourceMgrDefRetrievalICM RMDB
 *                    ex: java SResourceMgrDefRetrievalICM MY_RM icmnlsdb icmadmin password
 *                    ---------------------------------------------------------------------------
 *      PREREQUISITS: The specified reference name must be already defined, such as the default
 *                    "RMDB" specified during install.  If a new RM definition is to be 
 *                    configured, please refer to SResourceMgrDefCreationICM Sample.
 *                    ---------------------------------------------------------------------------
 *    FILES REQUIRED: SConnectDisconnectICM
 ************************************************************************************************/
public class SResourceMgrDefRetrievalICM{
    
    //-------------------------------------------------------------
    // Main
    //-------------------------------------------------------------
    /**
     * Run the Sample.
     * @param argv[] String Array containing arguments.  Optional arguments are <databse> <userName> <password>.
     */
    public static void main(String argv[]) throws DKException, Exception{
    
        // Defaults for connecting to the database.
        String database = SConnectDisconnectICM.DEFAULT_DATABASE;
        String userName = SConnectDisconnectICM.DEFAULT_USERNAME;
        String password = SConnectDisconnectICM.DEFAULT_PASSWORD;

        // RM Configuration Variables.  Used to store commandline arguments.
        String rmUserDefinedReferenceName  = "NOT_SPECIFIED";
        
        //------------------------------------------------------------
        // Checking for input parameters
        //--------------------------------------------------------------
        // Require all RM information be specified
        if (argv.length < 4) { // if not all LS Database arguments were specified, use defaults and report correct usage.
            System.out.println("Usage: " );
            System.out.println("  java SResourceMgrDefRetrievalICM <Existing RM User-Defined Reference Name>");
            System.out.println("                                   <database> <userName> <password>");
            System.out.println("");
            System.out.println("  NOTE: The specified reference name must be already defined, such as the");
            System.out.println("        default 'RMDB' specified during install.  If a new RM definition");
            System.out.println("        is to be configured, please refer to SResourceMgrDefCreationICM");
            System.out.println("        Sample.  This name should consist of all upper-case letters.");
            System.out.println(""); 
            System.out.println("  NOTE: Defaults will be used if the database, username, & password are not");
            System.out.println("        specified.  All others are required.");
            System.out.println("");
            System.out.println("  ex: java SResourceMgrDefRetrievalICM RMDB");
            System.out.println("  ex: java SResourceMgrDefRetrievalICM MY_RM icmnlsdb icmadmin password");
        }
        if((argv.length < 1) ){ // if every RM parameter not specified if going to attempt to setup, stop the program.
            System.out.println("======================================================");
            System.out.println("PROGRAM TERMINATED:  All RM Options Must Be Specified.");
            System.out.println("======================================================");
            return;
        } else if (argv.length != 4)
            System.out.println("  *** Some LS Database parameters not specified, using defaults..." );
        
        // Store commandline arguments.
        if (argv.length > 0) rmUserDefinedReferenceName = argv[0];

        // Store optional commandline arguments.
        if (argv.length > 1)  database = argv[1];
        if (argv.length > 2)  userName = argv[2];
        if (argv.length > 3)  password = argv[3];
        String ver = SConnectDisconnectICM.EIP_VERSION;
            
        System.out.println("============================================");
        System.out.println("IBM Information Integrator for Content  v"+ver);
        System.out.println("Sample Program:  SResourceMgrDefRetrievalICM");
        System.out.println("--------------------------------------------");
        System.out.println("           Database:  "+database);
        System.out.println("           UserName:  "+userName);
        System.out.println("  RM Reference Name:  "+rmUserDefinedReferenceName);       
        System.out.println("============================================\n");
        
        try{
            //-------------------------------------------------------------
            // Connect to datastore
            //-------------------------------------------------------------
            // See Sample SConnectDisconnectICM for more information
            System.out.println("Connecting to datastore (Database '"+database+"', UserName '"+userName+"')...");

                DKDatastoreICM dsICM = new DKDatastoreICM();  // Create new datastore object.
                dsICM.connect(database,userName,password,""); // Connect to the datastore.

            System.out.println("Connected to datastore (Database '"+dsICM.datastoreName()+"', UserName '"+dsICM.userName()+"').");

            //-------------------------------------------------------------
            // PLEASE READ
            //-------------------------------------------------------------
            // This sample will heavily utilize the wrapper functions provided
            // at the bottom of this sample.  These funcitons will be used to
            // demonstrate usage of concepts in this sample.

            //------------------------------------------------------------------
            // List Resource Managers
            //------------------------------------------------------------------
            // Listing / Printing Resource Managers found in the system
            System.out.println("Listing / Printing Resource Managers found in the system...");

                printAllResourceManagers(dsICM); // Uses wrapper at bottom of sample.

            System.out.println("Listing / Printing Resource Managers found in the system...");

            //-------------------------------------------------------------
            // Checking to See if RM Definition Already Setup
            //-------------------------------------------------------------
            // This sample will skip over operations that have already been completed
            // in a previous run of this sample or other program.
            //
            // Demonstrating using Wrapper at bottom of sample.
            System.out.println("Checking to see if RM Definition already setup...");

                if(isResourceManagerSetup(dsICM, rmUserDefinedReferenceName))
                    System.out.println("RM Definition '"+rmUserDefinedReferenceName+"' already setup.");
                else
                    System.out.println("RM Definition '"+rmUserDefinedReferenceName+"' has not yet been setup.");

            System.out.println("Completed checking to see if RM Definition already setup.");

            //-------------------------------------------------------------
            // Obtaining an RM Code for a RM Definition
            //-------------------------------------------------------------
            // Obtaining the RM Code that is used to specify properties of Items
            // and Item Types.
            System.out.println("Obtaining RM Code for RM Definition '"+rmUserDefinedReferenceName+"'...");            
            
                short rmCode = getResourceManagerIDCode(dsICM, rmUserDefinedReferenceName); // Use the wrapper function at bottom of sample.
            
            System.out.println("Obtained RM Code '"+rmCode+"' for RM Definition '"+rmUserDefinedReferenceName+"'.");            

            //------------------------------------------------------------------
            // Print Resource Manager Definition
            //------------------------------------------------------------------
            System.out.println("Printing Resource Manager Definition...");

                printResourceManagerDefinition(dsICM, rmUserDefinedReferenceName); // Uses wrapper at bottom of sample.

            System.out.println("Printed Resource Manager Definition.");

            //-------------------------------------------------------------
            // Disconnect from datastore & Destroy Reference
            //-------------------------------------------------------------
            System.out.println("Disconnecting from datastore & destroying reference...");            

                dsICM.disconnect();
                dsICM.destroy();

            System.out.println("Disconnected from datastore & destroying reference.");
        
            //-------------------------------------------------------------
            // Sample program completed without exception
            //-------------------------------------------------------------
            System.out.println("\n==========================================");
            System.out.println("Sample program completed.");
            System.out.println("==========================================\n");
        }
        //------------------------------------------------------------
        // Catch & Print Exceptions        
        //------------------------------------------------------------
        catch (DKException exc){
            SConnectDisconnectICM.printException(exc);  // Print the exception using the function listed below.
            throw(exc);
        } catch (Exception exc) {
            SConnectDisconnectICM.printException(exc);  // Print the exception using the function listed below.
            throw(exc);
        }
    }// end main


    //=================================================================
    // Wrapper Functions
    //=================================================================
    // The following are wrapper functions for functionality covered
    // in this sample.  These functions may be used by other samples.

    //******************************************************************
    // getResourceManagerIDCode
    //******************************************************************
   /**
    * Get the specified Resource Manager's ID Code for the specified user defined reference name.  This function should not be used to simply validate that the RM definition is setup.
    * @param dsICM  Connected DKDatastoreICM object.
    * @param rmUserDefinedReferenceName  The user-defined reference name to the one of the defined resource managers in the Library Server.  It should consist of all upper-case letters.
    * @return  Returns the ID Code for the defined resource manager.
    **/
    public static short getResourceManagerIDCode(DKDatastoreICM dsICM, String rmUserDefinedReferenceName) throws DKException, Exception{
        System.out.println("Obtaining Resource Manager ID Code for User Defined Reference Name of '"+rmUserDefinedReferenceName+"'.");

        // Obtain the RM Configuration Management Object
        DKDatastoreDefICM        dsDefICM = (DKDatastoreDefICM)dsICM.datastoreDef();                     // Get the Datastore Definition Object
        DKDatastoreAdminICM      dsAdmin  = (DKDatastoreAdminICM) dsDefICM.datastoreAdmin();             // Get the Datqastore Administration
        DKConfigurationMgmtICM   cfgMgmt  = (DKConfigurationMgmtICM) dsAdmin.configurationManagement();  // Get the configuration management.
        DKRMConfigurationMgmtICM rmCfg    = cfgMgmt.rmConfigurationMgmt();                               // Get the Resource Manager configuration.

        short rmCode;
        // Find the Resource Manager ID code for the specified resource manager.
        try {
            DKResourceMgrDefICM rmDef = rmCfg.retrieveResourceMgr(rmUserDefinedReferenceName);  // Get the RM Definition for the specified Resource Manager.
            rmCode = rmDef.getRMCode();                       // Get the ID Code for the specified RM.
            System.out.println("RM "+rmUserDefinedReferenceName+"'s ID Code Retrieved: " + rmCode);
        }
        catch (Exception e){ // if an exception was thrown, it is because there are none defined in the system.
            // IMPORTANT: Please do not use try-catch exception logic for program logic.  Use DKRMConfigurationMgmtICM.listResourceMgrs() to search available Resource Managers.
            System.out.println("An error has occured, most likely because the specified reference, '"+rmUserDefinedReferenceName+"', to a resource manager is not found.  Please do not use try-catch exception logic for program logic.  Use DKRMConfigurationMgmtICM.listResourceMgrs() to search available Resource Managers.");
            throw e;
        }
            
        return(rmCode);
    }

    //******************************************************************
    // isResourceManagerSetup
    //******************************************************************
   /**
    * Determines if the specified user defined reference name to a Resource Manager is setup.
    * @param dsICM  Connected DKDatastoreICM object.
    * @param rmUserDefinedReferenceName  The user-defined reference name to the one of the defined resource managers in the Library Server.  It should consist of all upper-case letters.
    * @return  Returns true if it is found, false if not.
    **/
    public static boolean isResourceManagerSetup(DKDatastoreICM dsICM, String rmUserDefinedReferenceName) throws DKException, Exception{
        System.out.println("Determining if Resource Manager referred to as '"+rmUserDefinedReferenceName+"' is Setup.");

        boolean found = false;  // Return value tracking if it was found.
        short rmCode = -1;

        // Obtain the RM Configuration Management Object
        DKDatastoreDefICM        dsDefICM = (DKDatastoreDefICM)dsICM.datastoreDef();                     // Get the Datastore Definition Object
        DKDatastoreAdminICM      dsAdmin  = (DKDatastoreAdminICM) dsDefICM.datastoreAdmin();             // Get the Datqastore Administration
        DKConfigurationMgmtICM   cfgMgmt  = (DKConfigurationMgmtICM) dsAdmin.configurationManagement();  // Get the configuration management.
        DKRMConfigurationMgmtICM rmCfg    = cfgMgmt.rmConfigurationMgmt();                               // Get the Resource Manager configuration.

        // Obtain a list of RM Definitions.
        dkCollection RMDefinitions = rmCfg.listResourceMgrs(); // Obtain the RM Definitions defined in the system.
        dkIterator iter = RMDefinitions.createIterator();      // create an iterator to search for the specified name.

        while(iter.more() && (found==false)){ // while there are more RM Definition objects & it has not been found, continue searching
            DKResourceMgrDefICM rmDef = (DKResourceMgrDefICM)iter.next(); // Move pointer to next element & return that object.
            if(rmDef.getName().compareTo(rmUserDefinedReferenceName)==0){ // Check to see if names match
                found = true;                 // mark it as found
                rmCode = rmDef.getRMCode();   // Get the RM Code to Print at End of Function
            }
        }

        if(found) // Report Status
            System.out.println("It is setup.  Found "+rmUserDefinedReferenceName+" with ID Code '"+rmCode+"'.");
        else
            System.out.println("It is not setup.  Did not find an ID Code for '"+rmUserDefinedReferenceName+"'.");

        return(found);
    }

    //******************************************************************
    // printAllResourceManagers
    //******************************************************************
   /**
    * Prints / Lists all Resource Managers defined in the system.
    * @param dsICM  Connected DKDatastoreICM object.
    **/
    public static void printAllResourceManagers(DKDatastoreICM dsICM) throws DKException, Exception{

        // Obtain the RM Configuration Management Object
        DKDatastoreDefICM        dsDefICM = (DKDatastoreDefICM)dsICM.datastoreDef();                     // Get the Datastore Definition Object
        DKDatastoreAdminICM      dsAdmin  = (DKDatastoreAdminICM) dsDefICM.datastoreAdmin();             // Get the Datqastore Administration
        DKConfigurationMgmtICM   cfgMgmt  = (DKConfigurationMgmtICM) dsAdmin.configurationManagement();  // Get the configuration management.
        DKRMConfigurationMgmtICM rmCfg    = cfgMgmt.rmConfigurationMgmt();                               // Get the Resource Manager configuration.

        // Obtain a list of RM Definitions.
        dkCollection RMDefinitions = rmCfg.listResourceMgrs(); // Obtain the RM Definitions defined in the system.
        dkIterator iter = RMDefinitions.createIterator(); // create an iterator to search for the specified name.

        System.out.println("Resource Managers Defined in System:");
        while(iter.more()){ // while there are more RM Definition objects & it has not been found, continue searching
            DKResourceMgrDefICM rmDef = (DKResourceMgrDefICM)iter.next(); // Move pointer to next element & return that object.
            System.out.println("     - "+ rmDef.getName()+" ("+ rmDef.getRMCode() +")"); // Print its name & code.
        }
    }

    //******************************************************************
    // printResourceManagerDefinition
    //******************************************************************
   /**
    * Print the specified Resource Manager's Definition for the specified user defined reference name. 
    * @param dsICM  Connected DKDatastoreICM object.
    * @param rmUserDefinedReferenceName  The user-defined reference name to the one of the defined resource managers in the Library Server.  It should consist of all upper-case letters.
    **/
    public static void printResourceManagerDefinition(DKDatastoreICM dsICM, String rmUserDefinedReferenceName) throws DKException, Exception{

        // Obtain the RM Configuration Management Object
        DKDatastoreDefICM        dsDefICM = (DKDatastoreDefICM)dsICM.datastoreDef();                     // Get the Datastore Definition Object
        DKDatastoreAdminICM      dsAdmin  = (DKDatastoreAdminICM) dsDefICM.datastoreAdmin();             // Get the Datqastore Administration
        DKConfigurationMgmtICM   cfgMgmt  = (DKConfigurationMgmtICM) dsAdmin.configurationManagement();  // Get the configuration management.
        DKRMConfigurationMgmtICM rmCfg    = cfgMgmt.rmConfigurationMgmt();                               // Get the Resource Manager configuration.

        DKResourceMgrDefICM rmDef = rmCfg.retrieveResourceMgr(rmUserDefinedReferenceName);  // Get the RM Definition for the specified Resource Manager.


        System.out.println("-----------------------------------------------");
        System.out.println("         Resource Manager Definition");
        System.out.println("-----------------------------------------------");
        System.out.println("     Reference Name:  "+rmDef.getName());
        System.out.println("            ID Code:  "+rmDef.getRMCode());
        System.out.println("       INET Address:  "+rmDef.getINETAddress());    
        switch(rmDef.getPlatform()){
            case DKConstantICM.RM_PLATFORM_WIN:
                System.out.println("           Platform:  Windows");
                break;
            case DKConstantICM.RM_PLATFORM_AIX:
                System.out.println("           Platform:  AIX");
                break;
            case DKConstantICM.RM_PLATFORM_SUN:
                System.out.println("           Platform:  Sun");
                break;
            case DKConstantICM.RM_PLATFORM_390:
                System.out.println("           Platform:  OS/390");
                break;
            default:
                System.out.println("           Platform:  Other ("+rmDef.getPlatform()+")");
                break;
        }//end switch on Platform
        System.out.println("         RM User ID:  "+rmDef.getRMUserid());
        System.out.println("         Created By:  "+rmDef.getUserid());
        System.out.println("     Token Duration:  "+rmDef.getTokenDuration());

        // Print Access Type Definition Information
        dkCollection accessTypeDefs = rmDef.getAccessType();    // Get a list of all access types available for this RM Definition.
        dkIterator iter = accessTypeDefs.createIterator();      // Create an iterator to go through all access types.
        System.out.println("Access Types:");
        while(iter.more()){ // while there are still Access Types to print, continue
            DKAccessTypeDef accessTypeDef = (DKAccessTypeDef) iter.next();  // Move pointer to next element & return that object.
            int    accessType = accessTypeDef.getAccessType();  // access type, HTTP, FTP, LSRM, etc.
            String accessData = accessTypeDef.getAccessData();
            int    port       = accessTypeDef.getPortNumber();  // The port at which the Resource Manager is running.

            System.out.println("               Name:  "+accessTypeDef.getName()); 
            System.out.println("        Description:  "+accessTypeDef.getDescription()); 
            System.out.println("               Port:  "+port);  // using value from DKAccessTypeDef retrieved above.

            switch(accessType){
                case DKConstantICM.RM_ACCESSTYPE_HTTP:
                    System.out.println("        Access Type:  HTTP");
                    break;
                case DKConstantICM.RM_ACCESSTYPE_FTP:
                    System.out.println("        Access Type:  FTP");
                    break;
                case DKConstantICM.RM_ACCESSTYPE_RTP:
                    System.out.println("        Access Type:  RTP");
                    break;
                case DKConstantICM.RM_ACCESSTYPE_JDBC:
                    System.out.println("        Access Type:  JDBC");
                    break;
                case DKConstantICM.RM_ACCESSTYPE_LSRM:
                    System.out.println("        Access Type:  LSRM");
                    break;
                case DKConstantICM.RM_ACCESSTYPE_HTTPS:
                    System.out.println("        Access Type:  HTTPS");
                    break;
                default:
                    System.out.println("        Access Type:  Other ("+accessType+")");
                    break;
            }//end switch on accessType
            System.out.println("");
        }//end while all Access Type Definitions

        System.out.println("-----------------------------------------------");

    }//end printResourceManagerDefinition

    //******************************************************************
    // printAdminDomainDef
    //******************************************************************
   /**
    * Prints the specified Admin Domain Definition.
    * @param adminDomainDef  dkAdminDomainDef to print.
    **/
    public static void printAdminDomainDef(dkAdminDomainDef adminDomainDef) throws DKException, Exception{

        System.out.println("---------------------------------------------------");
        System.out.println("              Admin Domain Information");
        System.out.println("---------------------------------------------------");
        if(adminDomainDef==null)
            System.out.println("               Info:  None Specified");
        else{
            System.out.println("               Name:  "+adminDomainDef.getName());
            System.out.println("        Description:  "+adminDomainDef.getDescription());
            System.out.println("                 ID:  "+adminDomainDef.getID());            
            System.out.println("       NLS Language:  "+adminDomainDef.getNLSLanguage());
            // RM Names
            String names[] = adminDomainDef.getResourceMgrNames();
            if(names==null)
                System.out.println("           RM Names:  null");
            else{
                System.out.println("           RM Names:  -");
                for(int i=0; i<names.length; i++) // print all RM Names
                    System.out.println("                      "+names[i]);
            }
            // SMS Collection Names
            names = adminDomainDef.getSMSCollectionNames();
            if(names==null)
                System.out.println("    SMS Coll. Names:  null");
            else{
                System.out.println("    SMS Coll. Names:  -");
               for(int i=0; i<names.length; i++) // print all SMS Collection Names
                    System.out.println("                      "+names[i]);
            }
            // ACL Names
            names = adminDomainDef.getACLNames();
            if(names==null)
                System.out.println("          ACL Names:  null");
            else{
                System.out.println("          ACL Names:  -");
                for(int i=0; i<names.length; i++) // print all ACL Names
                    System.out.println("                      "+names[i]);
            }
            // User Names            
            names = adminDomainDef.getUserNames();
            if(names==null)
                System.out.println("         User Names:  null");
            else{
                System.out.println("         User Names:  -");
                for(int i=0; i<names.length; i++) // print all User Names
                    System.out.println("                      "+names[i]);
            }
            // User Group Names            
            names = adminDomainDef.getUserGroupNames();
            if(names==null)
                System.out.println("   User Group Names:  null");
            else{
                System.out.println("   User Group Names:  -");
                for(int i=0; i<names.length; i++) // print all User Group Names
                    System.out.println("                      "+names[i]);
            }
            // Priv. Set Names
            names = adminDomainDef.getPrivSetNames();
            if(names==null)
                System.out.println("    Priv. Set Names:  -");
            else{
                System.out.println("    Priv. Set Names:  -");
                for(int i=0; i<names.length; i++) // print all Privilege Set Names
                    System.out.println("                      "+names[i]);
            }
        }//end adminDomainDef
    }
   
}//end class SResourceMgrDefRetrievalICM
              
